//---------------------------------------------------------------------------
#pragma hdrstop
#include "Data.h"
#include "EntryData.h"
#include "LoadList.h"
#include "TBLParser.h"
#include "Options.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//------------------------------------------------------------------------------
/*
############################################################################
This routine is the meeting point for many important functionalities:
it stores the name of the edited DAT file, loads a specific format
for it and, finally, initiates the binary parse through the DAT file.
############################################################################
*/
void CheckDATFile (String FileName) {
     if (FileExists(FileName)) {
        if (FileName.Pos('/') != 0) {
           for (int i=1;i<FileName.Length();i++) {
              if (FileName[i] == '/') {
                 FileName[i] = '\\';
              }
           }
        }
        TMemoryStream* WorkFile = new TMemoryStream();
        if (WorkFile != NULL) {
           WorkFile->LoadFromFile(FileName);
           for (int i=0;i<11;i++) {
               if (WorkFile->Size == GetDATSize(i)) {
                  GetDATObject(i)->SetDatName(FileName);
                  if (GetDATObject(i)->GetFormatLoaded() == false) {
                     GetDATObject(i)->LoadFormat(GetDATType(i));
                     GetDATObject(i)->SetFormatLoaded(true);
                  }
                  GetDATObject(i)->LoadData(WorkFile->Memory);
                  Form1->Editors->ActivePageIndex=i;
                  Form1->StatusBar->Panels->Items[0]->Text = FileName;
                  Form1->StatusBar->Update();
                  Form1->ChangeTab->Execute();
                  break;
               }
               else if (i==10){
                  MessageDlg(String("\""+FileName+"\" is not a valid DAT file that DatEdit can recognize."), mtError, TMsgDlgButtons() << mbOK, 0);
               }
           }
        }
        else {
           MessageDlg(String("\""+FileName+"\" cannot be opened."), mtError, TMsgDlgButtons() << mbOK, 0);
        }
        delete WorkFile;
    }
    else {
       MessageDlg("Target file does not exist!", mtError, TMsgDlgButtons() << mbOK, 0);
    }
};
//------------------------------------------------------------------------------
/*
############################################################################
Run the file output routine for a specific DAT file.
############################################################################
*/

void SaveDatFile () {
     if (GetDATObject(Form1->GetPage())->GetLoadState() == true) {
          if (GetDATObject(Form1->GetPage())->GetDefaultState() == true) {
             String DatType = GetDATType(Form1->GetPage());

             int DefaultOverwriteMode = int (MessageDlg(("Are you sure that you want to overwrite the default \""+DatType+".dat\" file?").c_str(), mtConfirmation, mbYesNoCancel, 0));

             if (DefaultOverwriteMode == 6) {
                GetDATObject(Form1->GetPage())->SaveData(GetDATObject(Form1->GetPage())->GetDatName());
                MessageDlg("File saved succesfully.", mtInformation, TMsgDlgButtons() << mbOK, 0);
             }
             else if (DefaultOverwriteMode == 7) {
                 Form1->SaveAs->Execute();
             }
          }
          else {
             GetDATObject(Form1->GetPage())->SaveData(GetDATObject(Form1->GetPage())->GetDatName());
             MessageDlg("File saved succesfully.", mtInformation, TMsgDlgButtons() << mbOK, 0);
          }
     }
     else if (GetDATObject(Form1->GetPage())->GetLoadState() == false) {
          MessageDlg("No DAT File of the current type was loaded. Load a DAT file first.", mtError, TMsgDlgButtons() << mbOK, 0);
     }
};
//------------------------------------------------------------------------------
/*
############################################################################
Open a given MPQ file and check it for DatEdit-compatible content
(DATs,TBLs and icons). When needed, call for the MPQ-importing routine.
############################################################################
*/
bool CheckMPQFile (String MPQFileName) {
    bool FileWasFound=false;
    void *MpqHandle,*FileHandle;

    for (int i=0;i<11;i++) {
        if (i<5) {
           Form1->SetTBLIsMPQImport(i,false);
        }
        GetDATObject(Form1->GetPage())->SetIsMPQImport(false);
    }
    Form1->SetIconIsMPQImport(false);
    Form1->SetMPQName("");

    ClearTemp();

    if (SFileOpenArchive(MPQFileName.c_str(),0,0,&MpqHandle)) {
       for (int i=10;i>=0;i--) {
           GetDATObject(i)->SetIsMPQImport(SFileOpenFile(String("arr\\"+GetDATType(i)+".dat").c_str(),&FileHandle));
           SFileCloseFile(FileHandle);
           if (FileWasFound == false) {
              FileWasFound = GetDATObject(i)->GetIsMPQImport();
           }
       }

       if (FileWasFound == true) {
          String FoundInMPQ;
          Form1->Options->Enabled = false;
          Form1->LoadDefault->Enabled = false;
          Form1->SetMPQName(MPQFileName);

          for (int i=10;i>=0;i--) {
              if (GetDATObject(i)->GetIsMPQImport() == true) {
     	         ImportFileFromMpq(MPQFileName,"arr\\"+GetDATType(i)+".dat");
                 CheckDATFile(Form1->GetMPQPath()+GetDATType(i)+".dat");
                 FoundInMPQ = FoundInMPQ+GetDATType(i)+".dat, ";

                 Form1->Bookmarks->Items[i]->Enabled = true;
                 Form1->Editors->Pages[i]->TabVisible = true;
                 Form1->Editors->Pages[i]->Enabled = true;
              }
              else if (GetDATObject(i)->GetIsMPQImport() == false) {
                   Form1->Editors->Pages[i]->TabVisible = false;
                   Form1->Editors->Pages[i]->Enabled = false;
                   Form1->Bookmarks->Items[i]->Enabled = false;
              }
          }
      
          if (SFileOpenFile(String("unit\\cmdbtns\\cmdicons.grp").c_str(),&FileHandle)) {
             ImportFileFromMpq(MPQFileName,"unit\\cmdbtns\\cmdicons.grp");
             Form1->SetIconIsMPQImport(true);
             Form1->SetMPQIcons(Form1->GetMPQPath()+"cmdicons.grp");
             FoundInMPQ = FoundInMPQ+"cmdicons.grp, ";
             if (GetDATObject(1)->GetIsMPQImport() == true) {
                Form1->SetEditorReload(1,true);
             }
             if (GetDATObject(5)->GetIsMPQImport() == true) {
                Form1->SetEditorReload(5,true);
             }
             if (GetDATObject(6)->GetIsMPQImport() == true) {
                Form1->SetEditorReload(6,true);
             }
          }
          SFileCloseFile(FileHandle);

          for (int i=0;i<5;i++) {
              if (i == 0) {
                 if (SFileOpenFile(String("rez\\stat_txt.tbl").c_str(),&FileHandle)) {
                    ImportFileFromMpq(MPQFileName,String("rez\\stat_txt.tbl"));
                    Form1->SetTBLIsMPQImport(0,true);
                    Form1->SetMPQTBL(0,String(Form1->GetMPQPath()+"stat_txt.tbl"));
                    FoundInMPQ = FoundInMPQ+GetTBLType(0)+".tbl, ";

                    Form1->SetEditorReload(0,true);
                    Form1->SetEditorReload(1,true);
                    Form1->SetEditorReload(5,true);
                    Form1->SetEditorReload(6,true);
                    Form1->SetEditorReload(10,true);
                 }
                 SFileCloseFile(FileHandle);
              }
              else {
                 if (SFileOpenFile(String("arr\\"+GetTBLType(i)+".tbl").c_str(),&FileHandle))  {
                    ImportFileFromMpq(MPQFileName,String("arr\\"+GetTBLType(i)+".tbl"));
                    Form1->SetTBLIsMPQImport(i,true);
                    Form1->SetMPQTBL(i,String(Form1->GetMPQPath()+GetTBLType(i)+".tbl"));
                    FoundInMPQ = FoundInMPQ+GetTBLType(i)+".tbl, ";
                    switch (i) {
                        case 1:
                             Form1->SetEditorReload(4,true);
                             break;
                        case 2:
                             Form1->SetEditorReload(8,true);
                             break;
                        case 3:
                             Form1->SetEditorReload(7,true);
                             break;
                        case 4:
                             Form1->SetEditorReload(9,true);
                             break;
                    }
                 }
                 SFileCloseFile(FileHandle);
              }
          }
          FoundInMPQ = FoundInMPQ.SubString(0,FoundInMPQ.Length()-2);
          MessageDlg(String("Imported the following files from "+MPQFileName+":\n"+FoundInMPQ), mtInformation, TMsgDlgButtons() << mbOK, 0);
       }
       else if (FileWasFound == false) {
          MessageDlg(String(MPQFileName+" does not contain any DAT files!"), mtError, TMsgDlgButtons() << mbOK, 0);
       }
    }
    else {
       return 0;
    }
    SFileCloseArchive(MpqHandle);
    if (Form1->GetPage() != 0) {
       Form1->Editors->Pages[Form1->GetPage()]->OnShow(Form1->Editors->Pages[Form1->GetPage()]);
    }
    else {
       Form1->UnitTabs->Pages[Form1->GetSubPage()]->OnShow(Form1->UnitTabs->Pages[Form1->GetSubPage()]);
    }
    Form1->ChangeTab->Execute();
    return 1;
};
//------------------------------------------------------------------------------
/*
############################################################################
Read a specific file from an MPQ archive into a buffer and pass it over
to the TEMP-directory-writing routine. Thanks to ShadowFlare for helping
me improve this.
############################################################################
*/
bool ImportFileFromMpq (String MpqName,String FileName) {
    void *FileHandle,*MpqHandle,*FileBuffer;
    unsigned long BytesRead;
    if (SFileOpenArchive(MpqName.c_str(),0,0,&MpqHandle)) {
       if (SFileOpenFile(FileName.c_str(),&FileHandle)) {
          int FileSize = SFileGetFileSize(FileHandle,NULL);
          FileBuffer = new char [FileSize];
          if (FileBuffer) {
             if (SFileReadFile(FileHandle,FileBuffer,FileSize,&BytesRead,NULL)) {
                SaveTempFile(&FileBuffer,FileSize,FileName);
             }
             delete FileBuffer;
          }
       }
       else {
            MessageDlg("DatEdit could not open the target file inside the MPQ archive.\nProbably it was deleted.", mtError, TMsgDlgButtons() << mbOK, 0);
       }
       SFileCloseFile(FileHandle);
    }
    else {
       MessageDlg("DatEdit could not open the target MPQ archive.\nProbably another application has blocked access to it.", mtError, TMsgDlgButtons() << mbOK, 0);
    }
    SFileCloseArchive(MpqHandle);
    return 1;
};
//------------------------------------------------------------------------------
/*
############################################################################
Save a buffer read from a file in an MPQ archive into a TEMP\MPQ directory
for further use by the program.
############################################################################
*/
void SaveTempFile(void** FileBuffer,int FileSize,String FileName) {
     if (!DirectoryExists(Form1->GetMPQPath())) {
        ForceDirectories(Form1->GetMPQPath());
     }
     int TempFile = FileCreate(Form1->GetMPQPath()+ExtractFileName(FileName));
     if (TempFile != -1) {
        FileWrite(TempFile,*FileBuffer,FileSize);
     }
     else {
          MessageDlg("Error creating temporary file.", mtError, TMsgDlgButtons() << mbOK, 0);
     }
     FileClose(TempFile);
};
//------------------------------------------------------------------------------
/*
############################################################################
Export a given file (DAT,TBL,cmdicons.grp) into a given MPQ archive,
using appropriate paramenters.
############################################################################
*/
void ExportToMpq(String MpqName,String FileName) {
     void *MpqHandle;
     String MPQDatType = ExtractFileName(FileName).SubString(0,FileName.Pos("."));
     int OpenParam,AddParam;

     if (FileExists(MpqName)) {
          OpenParam = 0x5;
          AddParam = 0x00000201;
     }
     else if (!FileExists(MpqName)) {
          OpenParam = 0x1;
          AddParam = 0x00000200;
     }
     MpqHandle = MpqOpenArchiveForUpdate(MpqName.c_str(),OpenParam,1024);
     if (FileName.Pos("dat") != 0) {
          MpqAddFileToArchive(MpqHandle, FileName.c_str(), ("arr\\"+MPQDatType).c_str(), AddParam);
     }
     else if (FileName.Pos("tbl") != 0) {
          if (FileName.Pos("txt") != 0) {
               MpqAddFileToArchive(MpqHandle, FileName.c_str(), ("rez\\"+MPQDatType).c_str(), AddParam);
          }
          else {
               MpqAddFileToArchive(MpqHandle, FileName.c_str(), ("arr\\"+MPQDatType).c_str(), AddParam);
          }
     }
     else if (FileName.Pos("cmdicons") != 0) {
          MpqAddFileToArchive(MpqHandle, FileName.c_str(), ("unit\\cmdbtns\\"+MPQDatType).c_str(), AddParam);
     }
     MpqCompactArchive(MpqHandle);
     MpqCloseUpdatedArchive(MpqHandle,0);
};

//------------------------------------------------------------------------------
/*
############################################################################
Return the left-side listing used with a specific editor.
############################################################################
*/
TTreeView *GetDATList (int PageIndex) {

switch (PageIndex) {
       case 0:
            return Form1->UnitList;
       case 1:
            return Form1->WeaponList;
       case 2:
            return Form1->FlingyList;
       case 3:
            return Form1->SpritesList;
       case 4:
            return Form1->ImageList;
       case 5:
            return Form1->UpgradeList;
       case 6:
            return Form1->TechList;
       case 7:
            return Form1->SoundList;
       case 8:
            return Form1->PortList;
       case 9:
            return Form1->MapList;
       case 10:
            return Form1->OrdList;
}
//This line is here just to remove the compiler warning.
return 0;
};
//------------------------------------------------------------------------------
/*
############################################################################
Return a specific DAT type as a String.
############################################################################
*/
String GetDATType (int PageIndex) {
     String DATType;
     switch (PageIndex) {
            case 0:
                 DATType = "units";
		 break;
	    case 1:
                 DATType = "weapons";
		 break;
	    case 2:
                 DATType = "flingy";
		 break;
	    case 3:
                 DATType = "sprites";
		 break;
	    case 4:
                 DATType = "images";
		 break;
	    case 5:
                 DATType = "upgrades";
		 break;
	    case 6:
                 DATType = "techdata";
		 break;
	    case 7:
                 DATType = "sfxdata";
		 break;
	    case 8:
                 DATType = "portdata";
		 break;
	    case 9:
                 DATType = "mapdata";
		 break;
	    case 10:
                 DATType = "orders";
		 break;
     }
     return DATType;
};
//------------------------------------------------------------------------------
/*
############################################################################
Return the default file size for a specific DAT type.
############################################################################
*/
int GetDATSize(int DATType) {

int Size;
switch (DATType) {
       case 0:
            Size = 19876;
            break;
       case 1:
            Size = 5460;
            break;
       case 2:
            Size = 3135;
            break;
       case 3:
            Size = 3229;
            break;
       case 4:
            Size = 37962;
            break;
       case 5:
            Size = 1281;
            break;
       case 6:
            Size = 836;
            break;
       case 7:
            Size = 10296;
            break;
       case 8:
            Size = 1320;
            break;
       case 9:
            Size = 260;
            break;
       case 10:
            Size = 4158;
            break;
}
return Size;
};
//------------------------------------------------------------------------------
/*
############################################################################
Return the list of offsets to different units.dat properties,
specific to a particular Units Editor subtab.
############################################################################
*/

int *GetTabMemoryOffs (int Tab) {

     int BasicTabOffs[23] = {6,7,8,17,18,19,20,23,24,25,26,27,41,42,43,45,46,47,48,49,50,51,53};
     int AdvTabOffs[4] = {1,2,10,22};
     int SoundTabOffs[7] = {29,30,31,32,33,34,35};
     int GfxTabOffs[9] = {0,3,4,5,9,37,38,39,40};
     int SETabOffs[6] = {11,36,44,45,52,54};
     int AITabOffs[7] = {12,13,14,15,16,21,28};

     int *TabMemoryOffs;
     switch (Tab) {
            case 0:
                 TabMemoryOffs = BasicTabOffs;
                 break;
            case 1:
                 TabMemoryOffs = AdvTabOffs;
                 break;
            case 2:
                 TabMemoryOffs = SoundTabOffs;
                 break;
            case 3:
                 TabMemoryOffs = GfxTabOffs;
                 break;
            case 4:
                 TabMemoryOffs = SETabOffs;
                 break;
            case 5:
                 TabMemoryOffs = AITabOffs;
                 break;
     }
     return TabMemoryOffs;
};
//------------------------------------------------------------------------------
/*
############################################################################
Return a string with the type of the DAT entries that are used
in a specific editor.
############################################################################
*/

String GetDataType (int PageIndex) {
    String DataType;
    switch (PageIndex) {
        case 0:
            DataType = "Units";
            break;
        case 1:
            DataType = "Weapons";
            break;
        case 5:
            DataType = "Upgrades";
            break;
        case 6:
            DataType = "Technologies";
            break;
    }
    return DataType;
};
//------------------------------------------------------------------------------
/*
############################################################################
Load the list of entry origins from the Origins.ini file, and return
a requested items from this list, i.e. the Origin value for a specific
DAT entry. Used with the "Sort by Origin" sorting method.

Origins should be read like the "Race" property:
0 - Zerg
1 - Terran
2 - Protoss
3 - Other (All)
############################################################################
*/

int GetOrigin (int Index, int PageIndex) {
    int Origin;
    String OriginList;
    TIniFile *OriginIni = new TIniFile(Form1->GetProgPath()+"default\\Origins.ini");
    OriginList = OriginIni->ReadString("Origins", GetDataType(PageIndex),"");

    Origin = OriginList.SubString(1+Index*2,1).ToInt();

    delete OriginIni;
    return Origin;
};
//------------------------------------------------------------------------------
/*
############################################################################
Load the list of entry suborigins from the Origins.ini file, and return
a requested items from this list. Suborigins are subgroups of the main
Origin types. Used with the "Sort by Origin" sorting method.

Suborigins should be read like this:
0-Ground Units
1-Air Units
2-Heroes
3-Buildings
4-Addons
5-Special Buildings
6-Special
7-Critters
8-Powerups
9-Resources

The group after the backslash is used by the "Other" (3) Origin type.
############################################################################
*/
int GetSubOrigin (int Index) {
    int Origin;
    if (Form1->TextBuf2->Lines->Count == 0) {
        TIniFile *OriginIni = new TIniFile(Form1->GetProgPath()+"default\\Origins.ini");
        Form1->TextBuf2->Lines->Add(OriginIni->ReadString("Origins", "UnitsSub",""));
        delete OriginIni;
    }
    Origin = String("0x"+Form1->TextBuf2->Lines->Strings[0].SubString(1+Index*2,1)).ToInt();
    return Origin;
};
//------------------------------------------------------------------------------
/*
############################################################################
Return a particular entry's Race property (thus used only for the
Upgrades and Tech editors). Used with the "Sort by Data" sorting
method.
############################################################################
*/

int GetSortData (int Index, int PageIndex) {
    int DataOrigin;
    int DataModulo;
    switch (PageIndex) {
           case 5:
                DataModulo = 9;
                break;
           case 6:
                DataModulo = 7;
                break;
    }
    DataOrigin = GetDATObject(PageIndex)->GetData()[Index*GetDATObject(PageIndex)->GetOffcount()+DataModulo];
    return DataOrigin;
};
//------------------------------------------------------------------------------
/*
############################################################################
Return the StarEdit race value of a units.dat entry, for
sorting purposes.

Used in Units Editor with the "Sort by Data" sorting method.
############################################################################
*/

int GetUnitsSortData (int Index) {
    int UnitsDataSort,UnitsDataSEGroup;
    UnitsDataSEGroup = GetDATObject(0)->GetData()[Index*GetDATObject(0)->GetOffcount()+45];
    UnitsDataSort = 3;
    if ((0x1&UnitsDataSEGroup) == 1) {
       UnitsDataSort = 0;
    }
    else if ((0x2&UnitsDataSEGroup)>>1 == 1) {
       UnitsDataSort = 1;
    }
    else if ((0x4&UnitsDataSEGroup)>>2 == 1) {
       UnitsDataSort = 2;
    }
    return UnitsDataSort;
};
//------------------------------------------------------------------------------
/*
############################################################################
Return a specific directory value (same as somewhere above) of
a units.dat entry, under specific conditions. This is the very core
of the "Sort by Data" sorting style. The numbers AND-ed with the 3 values
are property masks - consult the EntryData.cpp file for reference about
what do they exactly mean.

Directory reference:
0-Ground Units
1-Air Units
2-Heroes
3-Buildings
4-Addons
5-Special Buildings
6-Special
7-Critters
8-Powerups
9-Resources

Used in Units Editor with the "Sort by Data" sorting method.
############################################################################
*/

int GetUnitsSortSubData (int Index) {
    int UnitsDataOrigin,UnitsDataAdv,UnitsDataSEGroup,UnitsDataSEAvail;
    UnitsDataSEGroup = GetDATObject(0)->GetData()[Index*GetDATObject(0)->GetOffcount()+45];
    UnitsDataAdv = GetDATObject(0)->GetData()[Index*GetDATObject(0)->GetOffcount()+22];
    UnitsDataSEAvail = GetDATObject(0)->GetData()[Index*GetDATObject(0)->GetOffcount()+54];

    UnitsDataOrigin = 6;
    if ((0x80&UnitsDataSEGroup)>>7 == 1) {
        if ((0x10&UnitsDataSEGroup)>>4 == 1) {
            UnitsDataOrigin = 5;
        }
        if ((0x800000&UnitsDataAdv)>>23 == 1 ||  (0x800&UnitsDataAdv)>>11 ==1) {
           UnitsDataOrigin = 8;
        }
        if ((0x2000&UnitsDataAdv)>>13 == 1) {
           UnitsDataOrigin = 9;
        }
        if ((0x8&UnitsDataSEGroup)>>3 == 1) {
           UnitsDataOrigin = 7;
        }
    }
    else if ((0x8&UnitsDataSEGroup)>>3 == 1) {
        UnitsDataOrigin = 0;
        if ((0x4&UnitsDataAdv)>>2 == 1) {
           UnitsDataOrigin = 1;
        }
        if ((0x40&UnitsDataAdv)>>6 == 1) {
           UnitsDataOrigin = 2;
        }
    }
    else if (0x1&UnitsDataAdv == 1 || (0x20&UnitsDataAdv)>>5 == 1 || (0x10&UnitsDataSEGroup)>>4 == 1 || (0x20&UnitsDataSEGroup)>>5 == 1) {
        UnitsDataOrigin = 5;
        if ((0x8&UnitsDataSEAvail)>>3 == 1) {
           UnitsDataOrigin = 3;
        }
        if ((0x2&UnitsDataAdv)>>1 == 1) {
           UnitsDataOrigin = 4;
        }
    }
    return UnitsDataOrigin;
};
//------------------------------------------------------------------------------
/*
############################################################################
Return a specific TBL file's name, according to an Index variable.
I use this only for my own coder's convenience and the indexing is
completely arbitrary.
############################################################################
*/
String GetTBLType(int Index) {
String TBLType;
switch (Index) {
       case 0:
            TBLType = "stat_txt";
            break;
       case 1:
            TBLType = "images";
            break;
       case 2:
            TBLType = "portdata";
            break;
       case 3:
            TBLType = "sfxdata";
            break;
       case 4:
            TBLType = "mapdata";
            break;
}
return TBLType;
};
//------------------------------------------------------------------------------
/*
############################################################################
Run specific list-loading routines, depending on which
sorting method is selected for use with the current editor.
############################################################################
*/

void CheckSortingStyle(int SortingStyle, int PageIndex) {

if (Form1->GetListReload(PageIndex) == true || GetDATList(PageIndex)->Items->Count == 0) {
   switch (SortingStyle) {
       case 0:
            LoadListData(GetDATType(PageIndex),GetDATList(PageIndex));
            break;
       case 1:
            LoadListOrigin(GetDATType(PageIndex),GetDATList(PageIndex));
            break;
       case 2:
            break;
       case 3:
            LoadListID(GetDATType(PageIndex),GetDATList(PageIndex));
            break;
   }
}
Form1->SetListReload(PageIndex,false);
};
//------------------------------------------------------------------------------
/*
############################################################################
Empty out the DatEd's TEMP directory from program files.
############################################################################
*/
void ClearTemp() {
if (DirectoryExists(Form1->GetTempPath()) == true) {
   for (int i=0;i<11;i++) {
       if (FileExists(Form1->GetTempPath()+GetDATType(i)+".dat")) {
          DeleteFile(Form1->GetTempPath()+GetDATType(i)+".dat");
       }
       if (FileExists(Form1->GetMPQPath()+GetDATType(i)+".dat")) {
          DeleteFile(Form1->GetMPQPath()+GetDATType(i)+".dat");
       }
       if (i<5) {
          if (FileExists(Form1->GetMPQPath()+GetTBLType(i)+".tbl")) {
             DeleteFile(Form1->GetMPQPath()+GetTBLType(i)+".tbl");
          }
       }
   }
   if (FileExists(Form1->GetMPQPath()+"cmdicons.grp")) {
      DeleteFile(Form1->GetMPQPath()+"cmdicons.grp");
   }
   if (RemoveDir(Form1->GetMPQPath()) == false) {
      MessageDlg("DatEdit could not remove its MPQ temporary directory, because it contained unexpected files.", mtError, TMsgDlgButtons() << mbOK, 0);
   }
   if (RemoveDir(Form1->GetTempPath()) == false) {
      MessageDlg("DatEdit could not remove its TEMP directory, because it contained unexpected files.", mtError, TMsgDlgButtons() << mbOK, 0);
   }
}
};
//------------------------------------------------------------------------------
/*
############################################################################
Clear out all the MPQ-related data, enable the menus and reload default
DAT files.
############################################################################
*/
void ClearMPQImportData() {
   for (int i=10;i>=0;i--) {
       Form1->Editors->Pages[i]->TabVisible = true;
       Form1->Editors->Pages[i]->Enabled = true;
       Form1->Bookmarks->Items[i]->Enabled = true;
       if (GetDATObject(i)->GetIsMPQImport() == true) {
          CheckDATFile(Form1->GetProgPath()+"default\\"+GetDATType(i)+".dat");
          GetDATObject(i)->SetIsMPQImport(false);
       }
       if (i<5) {
          Form1->SetTBLIsMPQImport(i,false);
       }
   }
   Form1->SetMPQName("");
   Form1->Options->Enabled = true;
   Form1->LoadDefault->Enabled = true;
   Form1->SetIconIsMPQImport(false);
};
//------------------------------------------------------------------------------

