//---------------------------------------------------------------------------
#pragma hdrstop
#include "EntryData.h"
#include "Data.h"
#include "grpapi.h"
#include "Options.h"

/*
############################################################################
Declare the global scope variables used to handle different DAT
properties (X-Y pairs mostly), and -most importantly- the objects of the
FileData class, used to store the different DAT properties.
############################################################################
*/
FileData unit,weapon,flingy,sprite,image,upgrade,tech,sound,portrait,map,order;

unsigned int SEPlace,UnitSizeLeft,UnitSizeRight;
unsigned long DimSizeUp,DimSizeDown,SEPlaceHeight,AddonPlace,AddonY;
//---------------------------------------------------------------------------
#pragma package(smart_init)
//------------------------------------------------------------------------------
/*
############################################################################
Returns a specific FileData object (i.e. a specific DAT file data storage)
based on the currently opened page of the editor (passed as a parameter).
############################################################################
*/

FileData* GetDATObject (int PageIndex) {
     FileData *DATObject;
     switch (PageIndex) {
            case 0:
                 DATObject = &unit;
		 break;
	    case 1:
                 DATObject = &weapon;
		 break;
	    case 2:
                 DATObject = &flingy;
		 break;
	    case 3:
                 DATObject = &sprite;
		 break;
	    case 4:
                 DATObject = &image;
		 break;
	    case 5:
                 DATObject = &upgrade;
		 break;
	    case 6:
                 DATObject = &tech;
		 break;
	    case 7:
                 DATObject = &sound;
		 break;
	    case 8:
                 DATObject = &portrait;
		 break;
	    case 9:
                 DATObject = &map;
		 break;
	    case 10:
                 DATObject = &order;
		 break;
     }
     return DATObject;
};
//------------------------------------------------------------------------------
/*
############################################################################
Display the DAT properties in a particular editor. A particular function's
name indicates which editor (and which DAT file properties) is in question.
Editor-specific functions follow.
############################################################################
*/

void DisplayEntryData (int EntryID, int PageIndex) {
      switch (PageIndex) {
            case 0:
                 DisplayUnitData(EntryID);
		 break;
            case 1:
                 DisplayWeaponData(EntryID);
		 break;
            case 2:
                 DisplayFlingyData(EntryID);
                 break;
            case 3:
                 DisplaySpriteData(EntryID);
                 break;
            case 4:
		 DisplayImageData(EntryID);
                 break;
	    case 5:
                 DisplayUpgradeData(EntryID);
		 break;
            case 6:
                 DisplayTechData(EntryID);
                 break;
            case 7:
                 DisplaySoundData(EntryID);
		 break;
            case 8:
                 DisplayPortData(EntryID);
                 break;
            case 9:
		 DisplayMapData(EntryID);
                 break;
            case 10:
		 DisplayOrderData(EntryID);
		 break;
      }
};
//------------------------------------------------------------------------------

void DisplayUnitData (int EntryID) {
     if (Form1->GetSubPage() == 0) {
//#############BASIC################
	Form1->UnitShieldEnable->Checked = bool (unit.Data[EntryID*unit.offcount+6]);
	Form1->UnitSP->Text = unit.Data[EntryID*unit.offcount+7];
	Form1->UnitHP->Text = (unit.Data[EntryID*unit.offcount+8]/256);
	Form1->UnitArmUpgrade->ItemIndex = unit.Data[EntryID*unit.offcount+25];
	Form1->UnitArm->Text = unit.Data[EntryID*unit.offcount+27];
        Form1->UnitWeaponGround->Text = unit.Data[EntryID*unit.offcount+17];
	Form1->UnitGroundMax->Text = unit.Data[EntryID*unit.offcount+18];
        Form1->UnitWeaponAir->Text = unit.Data[EntryID*unit.offcount+19];
        Form1->UnitAirMax->Text = unit.Data[EntryID*unit.offcount+20];
	Form1->UnitSight->Text = unit.Data[EntryID*unit.offcount+24];
	Form1->UnitSize->ItemIndex = unit.Data[EntryID*unit.offcount+26];
	Form1->UnitSpaceReq->Text = unit.Data[EntryID*unit.offcount+48];
	Form1->UnitSpaceProv->Text = unit.Data[EntryID*unit.offcount+49];
	Form1->UnitScoreBuild->Text = unit.Data[EntryID*unit.offcount+50];
	Form1->UnitScoreDestroy->Text = unit.Data[EntryID*unit.offcount+51];
	Form1->UnitMinerals->Text = unit.Data[EntryID*unit.offcount+41];
	Form1->UnitVespene->Text = unit.Data[EntryID*unit.offcount+42];
	Form1->UnitTime->Text = unit.Data[EntryID*unit.offcount+43]/15;
	Form1->UnitIsBW->Checked = bool (unit.Data[EntryID*unit.offcount+53]);

        Form1->UnitSEGroupProtoss->Checked = bool((0x4&unit.Data[EntryID*unit.offcount+45])>>2);
        Form1->UnitSEGroupTerran->Checked = bool((0x2&unit.Data[EntryID*unit.offcount+45])>>1);
        Form1->UnitSEGroupZerg->Checked = bool(0x1&unit.Data[EntryID*unit.offcount+45]);

   	Form1->UnitSubRange->Text = unit.Data[EntryID*unit.offcount+23];
        Form1->UnitSupProv->Tag = (unit.Data[EntryID*unit.offcount+46]/2);
	Form1->UnitSupReq->Tag = (unit.Data[EntryID*unit.offcount+47]/2);
        if ((float(unit.Data[EntryID*unit.offcount+46])/2)-((unit.Data[EntryID*unit.offcount+46])/2) == 0.5) {
            Form1->UnitSupProv05->Checked = true;
        }
        else if ((float(unit.Data[EntryID*unit.offcount+46])/2)-((unit.Data[EntryID*unit.offcount+46])/2) != 0.5) {
            Form1->UnitSupProv05->Checked = false;
        }
        if ((float(unit.Data[EntryID*unit.offcount+47])/2)-((unit.Data[EntryID*unit.offcount+47])/2) == 0.5) {
            Form1->UnitSupReq05->Checked = true;
        }
        else if ((float(unit.Data[EntryID*unit.offcount+47])/2)-((unit.Data[EntryID*unit.offcount+47])/2) != 0.5) {
            Form1->UnitSupReq05->Checked = false;
        }
	Form1->UnitSupProv->Text = (unit.Data[EntryID*unit.offcount+46]/2);
	Form1->UnitSupReq->Text = (unit.Data[EntryID*unit.offcount+47]/2);
     }
//#############ADVANCED################
     else if (Form1->GetSubPage() == 1) {
        Form1->UnitAdvProd->Checked = bool((0x80000000&unit.Data[EntryID*unit.offcount+22])>>31);
        Form1->UnitAdvMech->Checked = bool((0x40000000&unit.Data[EntryID*unit.offcount+22])>>30);
        Form1->UnitAdvInv->Checked = bool((0x20000000&unit.Data[EntryID*unit.offcount+22])>>29);
        Form1->UnitAdvAttackRange->Checked = bool((0x10000000&unit.Data[EntryID*unit.offcount+22])>>28);
	Form1->UnitAdvAttackSight->Checked = bool((0x8000000&unit.Data[EntryID*unit.offcount+22])>>27);
	Form1->UnitAdvOverlayLarge->Checked = bool((0x4000000&unit.Data[EntryID*unit.offcount+22])>>26);
	Form1->UnitAdvOverlayMed->Checked = bool((0x2000000&unit.Data[EntryID*unit.offcount+22])>>25);
	Form1->UnitAdvIgnoreSupply->Checked = bool((0x1000000&unit.Data[EntryID*unit.offcount+22])>>24);
	Form1->UnitAdvPickup->Checked = bool((0x800000&unit.Data[EntryID*unit.offcount+22])>>23);
	Form1->UnitAdvPermaCloak->Checked = bool((0x400000&unit.Data[EntryID*unit.offcount+22])>>22);
	Form1->UnitAdvSpellcaster->Checked = bool((0x200000&unit.Data[EntryID*unit.offcount+22])>>21);
	Form1->UnitAdvBurrow->Checked = bool((0x100000&unit.Data[EntryID*unit.offcount+22])>>20);
	Form1->UnitAdvPsi->Checked = bool((0x80000&unit.Data[EntryID*unit.offcount+22])>>19);
	Form1->UnitAdvUnused->Checked = bool((0x40000&unit.Data[EntryID*unit.offcount+22])>>18);
	Form1->UnitAdvCreep->Checked = bool((0x20000&unit.Data[EntryID*unit.offcount+22])>>17);
	Form1->UnitAdvOrg->Checked = bool((0x10000&unit.Data[EntryID*unit.offcount+22])>>16);
        Form1->UnitAdvDetect->Checked = bool((0x8000&unit.Data[EntryID*unit.offcount+22])>>15);
        Form1->UnitAdvRobo->Checked = bool((0x4000&unit.Data[EntryID*unit.offcount+22])>>14);
        Form1->UnitAdvResContainer->Checked = bool((0x2000&unit.Data[EntryID*unit.offcount+22])>>13);
        Form1->UnitAdvResDepot->Checked = bool((0x1000&unit.Data[EntryID*unit.offcount+22])>>12);
	Form1->UnitAdvSingle->Checked = bool((0x800&unit.Data[EntryID*unit.offcount+22])>>11);
	Form1->UnitAdvTwoInEgg->Checked = bool((0x400&unit.Data[EntryID*unit.offcount+22])>>10);
        Form1->UnitAdvCloak->Checked = bool((0x200&unit.Data[EntryID*unit.offcount+22])>>9);
        Form1->UnitAdvAnimIdle->Checked = bool((0x100&unit.Data[EntryID*unit.offcount+22])>>8);
        Form1->UnitAdvReg->Checked = bool((0x80&unit.Data[EntryID*unit.offcount+22])>>7);
        Form1->UnitAdvHero->Checked = bool((0x40&unit.Data[EntryID*unit.offcount+22])>>6);
        Form1->UnitAdvFlyBuilding->Checked = bool((0x20&unit.Data[EntryID*unit.offcount+22])>>5);
        Form1->UnitAdvSub->Checked = bool((0x10&unit.Data[EntryID*unit.offcount+22])>>4);
        Form1->UnitAdvWorker->Checked = bool((0x8&unit.Data[EntryID*unit.offcount+22])>>3);
        Form1->UnitAdvFlyer->Checked = bool((0x4&unit.Data[EntryID*unit.offcount+22])>>2);
        Form1->UnitAdvAddon->Checked = bool((0x2&unit.Data[EntryID*unit.offcount+22])>>1);
        Form1->UnitAdvBuilding->Checked = bool(0x1&unit.Data[EntryID*unit.offcount+22]);

        Form1->UnitMov80->Checked = bool((0x80&unit.Data[EntryID*unit.offcount+10])>>7);
        Form1->UnitMov40->Checked = bool((0x40&unit.Data[EntryID*unit.offcount+10])>>6);
        Form1->UnitMov20->Checked = bool((0x20&unit.Data[EntryID*unit.offcount+10])>>5);
        Form1->UnitMov10->Checked = bool((0x10&unit.Data[EntryID*unit.offcount+10])>>4);
        Form1->UnitMov08->Checked = bool((0x8&unit.Data[EntryID*unit.offcount+10])>>3);
        Form1->UnitMov04->Checked = bool((0x4&unit.Data[EntryID*unit.offcount+10])>>2);
        Form1->UnitMov02->Checked = bool((0x2&unit.Data[EntryID*unit.offcount+10])>>1);
        Form1->UnitMov01->Checked = bool(0x1&unit.Data[EntryID*unit.offcount+10]);

	Form1->UnitUnk1->Text = unit.Data[EntryID*unit.offcount+44];
	Form1->UnitSub1->ItemIndex = unit.Data[EntryID*unit.offcount+1];
	Form1->UnitSub2->ItemIndex = unit.Data[EntryID*unit.offcount+2];
     }
//#############SOUND################
     else if (Form1->GetSubPage() == 2) {
	Form1->UnitSndWhatStart->Text = unit.Data[EntryID*unit.offcount+30];
	Form1->UnitSndWhatEnd->Text = unit.Data[EntryID*unit.offcount+31];
	if (EntryID >= 106) {
		Form1->UnitSndReady->Enabled = false;
		Form1->UnitSndReady->Text = "";
		Form1->UnitSndReady->Color = clBtnFace;
		Form1->UnitSndYesStart->Enabled = false;
		Form1->UnitSndYesStart->Text = "";
		Form1->UnitSndYesStart->Color = clBtnFace;
		Form1->UnitSndYesEnd->Enabled = false;
		Form1->UnitSndYesEnd->Text = "";
		Form1->UnitSndYesEnd->Color = clBtnFace;
		Form1->UnitSndAnnStart->Enabled = false;
		Form1->UnitSndAnnStart->Text = "";
		Form1->UnitSndAnnStart->Color = clBtnFace;
		Form1->UnitSndAnnEnd->Enabled = false;
		Form1->UnitSndAnnEnd->Text = "";
                Form1->UnitSndAnnEnd->Color = clBtnFace;

		Form1->UnitSndReadyList->Enabled = false;
		Form1->UnitSndReadyList->ItemIndex = -1;
		Form1->UnitSndReadyList->Color = clBtnFace;
		Form1->UnitSndYesStartList->Enabled = false;
		Form1->UnitSndYesStartList->ItemIndex = -1;
		Form1->UnitSndYesStartList->Color = clBtnFace;
		Form1->UnitSndYesEndList->Enabled = false;
		Form1->UnitSndYesEndList->ItemIndex = -1;
		Form1->UnitSndYesEndList->Color = clBtnFace;
		Form1->UnitSndAnnStartList->Enabled = false;
		Form1->UnitSndAnnStartList->ItemIndex = -1;
		Form1->UnitSndAnnStartList->Color = clBtnFace;
		Form1->UnitSndAnnEndList->Enabled = false;
		Form1->UnitSndAnnEndList->ItemIndex = -1;
		Form1->UnitSndAnnEndList->Color = clBtnFace;
	}
	else if (EntryID < 106) {
		Form1->UnitSndReady->Enabled = true;
		Form1->UnitSndReady->Color = clBlack;
		Form1->UnitSndReady->Text = unit.Data[EntryID*unit.offcount+29];
		Form1->UnitSndAnnStart->Enabled = true;
		Form1->UnitSndAnnStart->Color = clBlack;
		Form1->UnitSndAnnStart->Text = unit.Data[EntryID*unit.offcount+32];
		Form1->UnitSndAnnEnd->Enabled = true;
		Form1->UnitSndAnnEnd->Color = clBlack;
		Form1->UnitSndAnnEnd->Text = unit.Data[EntryID*unit.offcount+33];
		Form1->UnitSndYesStart->Enabled = true;
		Form1->UnitSndYesStart->Color = clBlack;
		Form1->UnitSndYesStart->Text = unit.Data[EntryID*unit.offcount+34];
		Form1->UnitSndYesEnd->Enabled = true;
		Form1->UnitSndYesEnd->Color = clBlack;
		Form1->UnitSndYesEnd->Text = unit.Data[EntryID*unit.offcount+35];

       		Form1->UnitSndReadyList->Enabled = true;
		Form1->UnitSndReadyList->Color = clBlack;
		Form1->UnitSndAnnStartList->Enabled = true;
		Form1->UnitSndAnnStartList->Color = clBlack;
		Form1->UnitSndAnnEndList->Enabled = true;
		Form1->UnitSndAnnEndList->Color = clBlack;
		Form1->UnitSndYesStartList->Enabled = true;
		Form1->UnitSndYesStartList->Color = clBlack;
		Form1->UnitSndYesEndList->Enabled = true;
		Form1->UnitSndYesEndList->Color = clBlack;
	}
     }
//#############GRAPHICS################
     else if (Form1->GetSubPage() == 3) {
       	Form1->UnitGfx->ItemIndex = unit.Data[EntryID*unit.offcount];
	Form1->UnitConst->ItemIndex = unit.Data[EntryID*unit.offcount+4];
	Form1->UnitAnimLevel->ItemIndex = unit.Data[EntryID*unit.offcount+9];
        if (unit.Data[EntryID*unit.offcount+40] == 0xFFFF) {
                Form1->UnitPort->ItemIndex = 220;
        }
        else if (unit.Data[EntryID*unit.offcount+40] < 220) {
                Form1->UnitPort->ItemIndex = unit.Data[EntryID*unit.offcount+40];
        }
	Form1->UnitDirection->Text = unit.Data[EntryID*unit.offcount+5];
	if (EntryID < 106 || EntryID > 201) {
		Form1->UnitInfest->ItemIndex = -1;
		Form1->UnitInfest->Enabled = false;
                Form1->JumpToUnitInfest->Enabled = false;
                Form1->UnitAddPosX->Enabled = false;
                Form1->UnitAddPosY->Enabled = false;
                Form1->UnitInfest->Color = clBtnFace;
                Form1->UnitAddPosX->Color = clBtnFace;
                Form1->UnitAddPosY->Color = clBtnFace;
                Form1->UnitAddPosX->Text = "";
                Form1->UnitAddPosY->Text = "";
	}
	else if (EntryID >= 106 && EntryID <=201) {
		Form1->UnitInfest->Enabled = true;
                Form1->JumpToUnitInfest->Enabled = true;
                Form1->UnitAddPosX->Enabled = true;
                Form1->UnitAddPosY->Enabled = true;
                Form1->UnitAddPosX->Color = clBlack;
                Form1->UnitAddPosY->Color = clBlack;
       		Form1->UnitInfest->Color  = clBlack;
		Form1->UnitInfest->ItemIndex = unit.Data[EntryID*unit.offcount+3];
                AddonPlace = unit.Data[EntryID*unit.offcount+37];
                Form1->UnitAddPosY->Text = (unsigned short) (AddonPlace>>16);
                Form1->UnitAddPosX->Text = (unsigned short) AddonPlace;
	}
	UnitSizeLeft = unit.Data[EntryID*unit.offcount+38];
	UnitSizeRight = unit.Data[EntryID*unit.offcount+39];
	Form1->UnitDimLeft->Text = (unsigned short) (UnitSizeLeft);
	Form1->UnitDimUp->Text = (unsigned short) (UnitSizeLeft>>16);
	Form1->UnitDimRight->Text = (unsigned short) (UnitSizeRight);
	Form1->UnitDimDown->Text = (unsigned short) (UnitSizeRight>>16);
        if (OptionsBox->ShowUnitSizePreview->Checked == true) {
             DrawUnitSize(unit.Data[EntryID*unit.offcount+38],unit.Data[EntryID*unit.offcount+39]);
        }
     }
//#############STAREDIT################
     else if (Form1->GetSubPage() == 4) {
	SEPlace = unit.Data[EntryID*unit.offcount+36];
	Form1->UnitSEPlaceWidth->Text = (unsigned short) (SEPlace);
	Form1->UnitSEPlaceHeight->Text = (unsigned short) (SEPlace>>16);
	Form1->UnitMapString->Text = unit.Data[EntryID*unit.offcount+52];
      	Form1->UnitRank->Text = unit.Data[EntryID*unit.offcount+11];

        Form1->UnitSEGroupNeutral->Checked = bool((0x80&unit.Data[EntryID*unit.offcount+45])>>7);
        Form1->UnitSEGroupInd->Checked = bool((0x40&unit.Data[EntryID*unit.offcount+45])>>6);
        Form1->UnitSEGroupFactory->Checked = bool((0x20&unit.Data[EntryID*unit.offcount+45])>>5);
        Form1->UnitSEGroupBuilding->Checked = bool((0x10&unit.Data[EntryID*unit.offcount+45])>>4);
        Form1->UnitSEGroupMen->Checked = bool((0x8&unit.Data[EntryID*unit.offcount+45])>>3);


	Form1->UnitSE0x8000->Checked = ((0x8000&unit.Data[EntryID*unit.offcount+54])>>15);
	Form1->UnitSE0x4000->Checked = ((0x4000&unit.Data[EntryID*unit.offcount+54])>>14);
	Form1->UnitSE0x2000->Checked = ((0x2000&unit.Data[EntryID*unit.offcount+54])>>13);
       	Form1->UnitSE0x1000->Checked = ((0x1000&unit.Data[EntryID*unit.offcount+54])>>12);
	Form1->UnitSE0x800->Checked = ((0x800&unit.Data[EntryID*unit.offcount+54])>>11);
	Form1->UnitSE0x400->Checked = ((0x400&unit.Data[EntryID*unit.offcount+54])>>10);
	Form1->UnitSEBW->Checked = ((0x200&unit.Data[EntryID*unit.offcount+54])>>9);
        Form1->UnitSELoc->Checked = ((0x100&unit.Data[EntryID*unit.offcount+54])>>8);
	Form1->UnitSEUnitHero->Checked = ((0x80&unit.Data[EntryID*unit.offcount+54])>>7);
	Form1->UnitSENonLoc->Checked = ((0x40&unit.Data[EntryID*unit.offcount+54])>>6);
	Form1->UnitSEDoodadState->Checked = ((0x20&unit.Data[EntryID*unit.offcount+54])>>5);
	Form1->UnitSEAllRaces->Checked = ((0x10&unit.Data[EntryID*unit.offcount+54])>>4);
        Form1->UnitSEPlayerSet->Checked = ((0x8&unit.Data[EntryID*unit.offcount+54])>>3);
        Form1->UnitSEBriefing->Checked = ((0x4&unit.Data[EntryID*unit.offcount+54])>>2);
        Form1->UnitSEListing->Checked = ((0x2&unit.Data[EntryID*unit.offcount+54])>>1);
	Form1->UnitSENonNeutral->Checked = (0x1&unit.Data[EntryID*unit.offcount+54]);

     }
//#############AI ACTIONS################
     else if (Form1->GetSubPage() == 5) {
	Form1->UnitAICompIdle->ItemIndex = unit.Data[EntryID*unit.offcount+12];
	Form1->UnitAIHumanIdle->ItemIndex = unit.Data[EntryID*unit.offcount+13];
	Form1->UnitAIReturn->ItemIndex = unit.Data[EntryID*unit.offcount+14];
	Form1->UnitAIAttackUnit->ItemIndex = unit.Data[EntryID*unit.offcount+15];
	Form1->UnitAIAttackMove->ItemIndex = unit.Data[EntryID*unit.offcount+16];
        if (unit.Data[EntryID*unit.offcount+28] < 7) {
                Form1->UnitAIRightClick->ItemIndex = unit.Data[EntryID*unit.offcount+28];
        }
        else if (unit.Data[EntryID*unit.offcount+28] >= 7) {
                Form1->UnitAIRightClick->ItemIndex = 7;
        }
        Form1->UnitAIInternal->Text =unit.Data[EntryID*unit.offcount+21];
     }
};

void DisplayWeaponData (int EntryID) {
	Form1->WeapLabel->ItemIndex = weapon.Data[EntryID*weapon.offcount];
	Form1->WeapGraphics->Text = weapon.Data[EntryID*weapon.offcount+1];
	Form1->WeapUnused->ItemIndex = weapon.Data[EntryID*weapon.offcount+2];

        Form1->WeapTarget100->Checked = ((0x100&weapon.Data[EntryID*weapon.offcount+3])>>8);
	Form1->WeapTarget080->Checked = ((0x80&weapon.Data[EntryID*weapon.offcount+3])>>7);
	Form1->WeapTarget040->Checked = ((0x40&weapon.Data[EntryID*weapon.offcount+3])>>6);
	Form1->WeapTarget020->Checked = ((0x20&weapon.Data[EntryID*weapon.offcount+3])>>5);
	Form1->WeapTarget010->Checked = ((0x10&weapon.Data[EntryID*weapon.offcount+3])>>4);
	Form1->WeapTarget008->Checked = ((0x8&weapon.Data[EntryID*weapon.offcount+3])>>3);
	Form1->WeapTarget004->Checked = ((0x4&weapon.Data[EntryID*weapon.offcount+3])>>2);
        Form1->WeapTarget002->Checked = ((0x2&weapon.Data[EntryID*weapon.offcount+3])>>1);
        Form1->WeapTarget001->Checked = (0x1&weapon.Data[EntryID*weapon.offcount+3]);

	Form1->WeapRangeMin->Text = (weapon.Data[EntryID*weapon.offcount+4]/16);
	Form1->WeapRangeMax->Text = (weapon.Data[EntryID*weapon.offcount+5]/16);
	Form1->WeapUpgrade->ItemIndex = weapon.Data[EntryID*weapon.offcount+6];
	Form1->WeapType->ItemIndex = weapon.Data[EntryID*weapon.offcount+7];
	Form1->WeapBehaviour->ItemIndex = weapon.Data[EntryID*weapon.offcount+8];
	switch (weapon.Data[EntryID*weapon.offcount+9]) {
	       case 0x1E:
		    Form1->WeapMissile->ItemIndex=0;
		    break;
	       case 0x3C:
		    Form1->WeapMissile->ItemIndex=1;
		    break;
	       case 0xFF:
		    Form1->WeapMissile->ItemIndex=2;
		    break;
	}
	Form1->WeapExplosion->ItemIndex = weapon.Data[EntryID*weapon.offcount+10];
	Form1->WeapInner->Text = weapon.Data[EntryID*weapon.offcount+11];
	Form1->WeapMed->Text = weapon.Data[EntryID*weapon.offcount+12];
	Form1->WeapOuter->Text = weapon.Data[EntryID*weapon.offcount+13];
	Form1->WeapAmount->Text = weapon.Data[EntryID*weapon.offcount+14];
	Form1->WeapBonus->Text = weapon.Data[EntryID*weapon.offcount+15];
	Form1->WeapCooldown->Text = weapon.Data[EntryID*weapon.offcount+16];
	Form1->WeapFactor->Text = weapon.Data[EntryID*weapon.offcount+17];

	Form1->WeapAttackAngle->Text = weapon.Data[EntryID*weapon.offcount+18];
        Form1->WeapLaunchSpin->Text = weapon.Data[EntryID*weapon.offcount+19];
	Form1->WeapForwardOffset->Text = weapon.Data[EntryID*weapon.offcount+20];
	Form1->WeapUpwardOffset->Text = weapon.Data[EntryID*weapon.offcount+21];

	Form1->WeapError->ItemIndex = weapon.Data[EntryID*weapon.offcount+22];
  	Form1->WeapIcon->Text = weapon.Data[EntryID*weapon.offcount+23];
        DrawIcon(weapon.Data[EntryID*weapon.offcount+23],Form1->WeapIconPrev);
};

void DisplayFlingyData (int EntryID) {
	Form1->FlingySprite->Text = flingy.Data[EntryID*flingy.offcount];
	Form1->FlingySpeed->Text = flingy.Data[EntryID*flingy.offcount+1];
	Form1->FlingyAcceleration->Text = flingy.Data[EntryID*flingy.offcount+2];
	Form1->FlingyHaltDist->Text = flingy.Data[EntryID*flingy.offcount+3];
	Form1->FlingyTurnRad->Text = flingy.Data[EntryID*flingy.offcount+4];
	Form1->FlingyUnk->Text = flingy.Data[EntryID*flingy.offcount+5];
	Form1->FlingyMoveControl->ItemIndex = flingy.Data[EntryID*flingy.offcount+6];
};

void DisplaySpriteData (int EntryID) {
	Form1->SpriteFile->Text = sprite.Data[EntryID*sprite.offcount];
        Form1->SpriteUnk1->Checked = bool (sprite.Data[EntryID*sprite.offcount+2]);
        Form1->SpriteVisible->Checked = bool (sprite.Data[EntryID*sprite.offcount+3]);
        if (EntryID < 130) {
	   Form1->SpriteHPBar->Text = "";
	   Form1->SpriteHPBar->Enabled = false;
	   Form1->SpriteHPBar->Color = clBtnFace;
           Form1->SpriteSelCircle->Text = "";
	   Form1->SpriteSelCircle->Enabled = false;
	   Form1->SpriteSelCircle->Color = clBtnFace;
           Form1->SpriteCircleSize->ItemIndex = -1;
	   Form1->SpriteCircleSize->Enabled = false;
	   Form1->SpriteCircleSize->Color = clBtnFace;
	   Form1->SpriteCircleOff->Text = "";
           Form1->SpriteCircleOff->Enabled = false;
           Form1->SpriteCircleOff->Color = clBtnFace;
        }
        else if (EntryID >= 130) {
           Form1->SpriteHPBar->Enabled = true;
           Form1->SpriteHPBar->Color = clBlack;
      	   Form1->SpriteSelCircle->Enabled = true;
	   Form1->SpriteSelCircle->Color = clBlack;
	   Form1->SpriteCircleSize->Enabled = true;
           Form1->SpriteCircleSize->Color = clBlack;
	   Form1->SpriteCircleOff->Enabled = true;
	   Form1->SpriteCircleOff->Color  = clBlack;
	   Form1->SpriteHPBar->Text = sprite.Data[(EntryID)*sprite.offcount+1];
           Form1->SpriteSelCircle->Text = sprite.Data[(EntryID)*sprite.offcount+4];
	   Form1->SpriteCircleSize->ItemIndex = Form1->SpriteSelCircle->Text.ToInt();
	   Form1->SpriteCircleOff->Text = sprite.Data[(EntryID)*sprite.offcount+5];
        }
};

void DisplayImageData (int EntryID) {
	Form1->ImgGRP->Text = image.Data[EntryID*image.offcount];
	Form1->ImgIscriptID->Text = image.Data[EntryID*image.offcount+7];

        Form1->ImgGfxTurns->Checked = bool (image.Data[EntryID*image.offcount+1]);
        Form1->ImgClickable->Checked = bool (image.Data[EntryID*image.offcount+2]);
	Form1->ImgUseIscript->Checked = bool (image.Data[EntryID*image.offcount+3]);
	Form1->ImgDrawCloaked->Checked = bool (image.Data[EntryID*image.offcount+4]);
	switch (image.Data[EntryID*image.offcount+5]) {
               case 0:
                    Form1->ImgPalette->ItemIndex=0;
                    break;
               case 1:
                    Form1->ImgPalette->ItemIndex=1;
                    break;
               case 3:
                    Form1->ImgPalette->ItemIndex=2;
                    break;
               case 8:
                    Form1->ImgPalette->ItemIndex=3;
                    break;
               case 9:
                    Form1->ImgPalette->ItemIndex=4;
                    break;
               case 10:
                    Form1->ImgPalette->ItemIndex=5;
                    break;
               case 17:
		    Form1->ImgPalette->ItemIndex=6;
                    break;
        }
	Form1->ImgPalSpec->ItemIndex = image.Data[EntryID*image.offcount+6];

        if (image.Data[EntryID*image.offcount+7] < Form1->ImgIscriptIDList->Items->Count) {
               Form1->ImgIscriptIDList->ItemIndex = image.Data[EntryID*image.offcount+7];
        }
        else if (image.Data[EntryID*image.offcount+7] >= Form1->ImgIscriptIDList->Items->Count) {
               Form1->ImgIscriptIDList->ItemIndex = -1;
        }

        Form1->ImgShield->Text = image.Data[EntryID*image.offcount+8];
        switch (Form1->ImgShield->Text.ToInt()) {
               case 0:
                    Form1->ImgShieldOL->ItemIndex=0;
                    break;
               case 2:
                    Form1->ImgShieldOL->ItemIndex=2;
                    break;
               case 133:
                    Form1->ImgShieldOL->ItemIndex=1;
                    break;
               case 184:
                    Form1->ImgShieldOL->ItemIndex=3;
                    break;
        }
	Form1->ImgOL1->Text = image.Data[EntryID*image.offcount+9];
	Form1->ImgOL2->Text = image.Data[EntryID*image.offcount+10];
	Form1->ImgOL3->Text = image.Data[EntryID*image.offcount+11];
	Form1->ImgOL4->Text = image.Data[EntryID*image.offcount+12];
	Form1->ImgOL5->Text = image.Data[EntryID*image.offcount+13];
};

void DisplayUpgradeData (int EntryID) {
	Form1->UpgMinerals->Text = upgrade.Data[EntryID*upgrade.offcount];
	Form1->UpgFactorMinerals->Text = upgrade.Data[EntryID*upgrade.offcount+1];
	Form1->UpgVespene->Text = upgrade.Data[EntryID*upgrade.offcount+2];
	Form1->UpgFactorVespene->Text = upgrade.Data[EntryID*upgrade.offcount+3];
	Form1->UpgTime->Text = upgrade.Data[EntryID*upgrade.offcount+4]/15;
	Form1->UpgFactorTime->Text = upgrade.Data[EntryID*upgrade.offcount+5]/15;
	Form1->UpgUnk->Text = upgrade.Data[EntryID*upgrade.offcount+6];
	Form1->UpgIcon->Text = upgrade.Data[EntryID*upgrade.offcount+7];
        DrawIcon(upgrade.Data[EntryID*upgrade.offcount+7],Form1->UpgIconPrev);
	Form1->UpgLabel->Text = upgrade.Data[EntryID*upgrade.offcount+8];
	Form1->UpgRace->ItemIndex = upgrade.Data[EntryID*upgrade.offcount+9];
	Form1->UpgRepeats->Text = upgrade.Data[EntryID*upgrade.offcount+10];
	Form1->UpgIsBW->Checked = bool (upgrade.Data[EntryID*upgrade.offcount+11]);
};

void DisplayTechData (int EntryID) {
	Form1->TechMinerals->Text = tech.Data[EntryID*tech.offcount];
	Form1->TechVespene->Text = tech.Data[EntryID*tech.offcount+1];
	Form1->TechTime->Text = tech.Data[EntryID*tech.offcount+2]/15;
	Form1->TechMana->Text = tech.Data[EntryID*tech.offcount+3];
	Form1->TechUnk->Text = tech.Data[EntryID*tech.offcount+4];
	Form1->TechIcon->Text = tech.Data[EntryID*tech.offcount+5];
        DrawIcon(tech.Data[EntryID*tech.offcount+5],Form1->TechIconPrev);
        Form1->TechLabel->Text = tech.Data[EntryID*tech.offcount+6];
	Form1->TechRace->ItemIndex = tech.Data[EntryID*tech.offcount+7];
	Form1->TechUnused->Checked = bool (tech.Data[EntryID*tech.offcount+8]);
	Form1->TechBW->Checked = bool (tech.Data[EntryID*tech.offcount+9]);
};
void DisplaySoundData (int EntryID) {
	Form1->SoundFile->Text = sound.Data[EntryID*sound.offcount];
	Form1->SoundUnk1->Text = sound.Data[EntryID*sound.offcount+1];
	Form1->SoundUnk2->Text = sound.Data[EntryID*sound.offcount+2];
	Form1->SoundUnk3->Text = sound.Data[EntryID*sound.offcount+3];
	Form1->SoundUnk4->Text = sound.Data[EntryID*sound.offcount+4];
};

void DisplayPortData (int EntryID) {
	Form1->PortFile->Text = portrait.Data[EntryID*portrait.offcount];
	Form1->PortSMKChange->Text = portrait.Data[(EntryID*portrait.offcount)+1];
	Form1->PortUnk1->Text = portrait.Data[(EntryID*portrait.offcount)+2];
};

void DisplayMapData (int EntryID) {
        Form1->MapFile->Text = map.Data[EntryID];
};

void DisplayOrderData (int EntryID) {
	Form1->OrdLabel->ItemIndex = order.Data[EntryID*order.offcount];
	Form1->OrdWeapTarg->Checked = bool (order.Data[EntryID*order.offcount+1]);
	Form1->OrdUnk2->Checked = bool (order.Data[EntryID*order.offcount+2]);
	Form1->OrdUnk3->Checked = bool (order.Data[EntryID*order.offcount+3]);
	Form1->OrdUnk4->Checked = bool (order.Data[EntryID*order.offcount+4]);
	Form1->OrdUnk5->Checked = bool (order.Data[EntryID*order.offcount+5]);
	Form1->OrdInterupt->Checked = bool (order.Data[EntryID*order.offcount+6]);
	Form1->OrdUnk7->Checked = bool (order.Data[EntryID*order.offcount+7]);
	Form1->OrdQueue->Checked = bool (order.Data[EntryID*order.offcount+8]);
	Form1->OrdUnk9->Checked = bool (order.Data[EntryID*order.offcount+9]);
	Form1->OrdUnk10->Checked = bool (order.Data[EntryID*order.offcount+10]);
	Form1->OrdUnk11->Checked = bool (order.Data[EntryID*order.offcount+11]);
	Form1->OrdUnk12->Checked = bool (order.Data[EntryID*order.offcount+12]);
	Form1->OrdWeapon->ItemIndex = order.Data[EntryID*order.offcount+13];
	Form1->OrdTech->ItemIndex = order.Data[EntryID*order.offcount+14];
	Form1->OrdAnimation->ItemIndex = order.Data[EntryID*order.offcount+15];
	Form1->OrdHighlight->Text = order.Data[EntryID*order.offcount+16];

	Form1->OrdUnk->Text = order.Data[EntryID*order.offcount+17];
	Form1->OrdEmulate->ItemIndex = order.Data[EntryID*order.offcount+18];
};
//------------------------------------------------------------------------------
/*
############################################################################
Keep track of the changes of the particular DAT properties in the
interface of a particular editor and apply the changes to the data stored
in the memory. A particular function's name indicates which editor
(and thus which DAT file properties) is in question.
############################################################################
*/

void ChangeUnitData (unsigned int value,int EntryID,TObject* Target) {
//#############BASIC################
if (Form1->GetSubPage() == 0) {
	if (Target ==Form1->UnitShieldEnable) {
		unit.Data[EntryID*unit.offcount+6] = bool (value);
	}
	else if (Target ==Form1->UnitSP) {
		unit.Data[EntryID*unit.offcount+7] = (unsigned short) (value);
	}
	else if (Target ==Form1->UnitHP) {
		unit.Data[EntryID*unit.offcount+8] = (unsigned long) (value*256);
	}
	else if (Target ==Form1->UnitArmUpgrade) {
		unit.Data[EntryID*unit.offcount+25] = (unsigned char) (value);
	}
	else if (Target ==Form1->UnitArm) {
		unit.Data[EntryID*unit.offcount+27] = (unsigned char) (value);
	}
        else if (Target == Form1->UnitWeaponGround) {
	     unit.Data[EntryID*unit.offcount+17] = (unsigned char) (value);
             if ((unsigned char) (value) < Form1->UnitWeaponGroundList->Items->Count) {
                Form1->UnitWeaponGroundList->ItemIndex = value;
             }
             else
                Form1->UnitWeaponGroundList->ItemIndex = -1;
   	}
	else if (Target ==Form1->UnitWeaponGroundList) {
                Form1->UnitWeaponGround->Text = value;
	}
	else if (Target ==Form1->UnitGroundMax) {
		unit.Data[EntryID*unit.offcount+18] = (unsigned char) (value);
	}
        else if (Target == Form1->UnitWeaponAir) {
	     unit.Data[EntryID*unit.offcount+19] = (unsigned char) (value);
             if ((unsigned char) (value) < Form1->UnitWeaponAirList->Items->Count) {
                Form1->UnitWeaponAirList->ItemIndex = value;
             }
             else
                Form1->UnitWeaponAirList->ItemIndex = -1;
   	}
	else if (Target ==Form1->UnitWeaponAirList) {
                Form1->UnitWeaponAir->Text = value;
	}
	else if (Target ==Form1->UnitAirMax) {
		unit.Data[EntryID*unit.offcount+20] = (unsigned char) (value);
	}
	else if (Target ==Form1->UnitSight) {
		unit.Data[EntryID*unit.offcount+24] = (unsigned char) (value);
	}
	else if (Target ==Form1->UnitSize) {
		unit.Data[EntryID*unit.offcount+26] = (unsigned char) (value);
	}
	else if (Target ==Form1->UnitSpaceReq) {
		unit.Data[EntryID*unit.offcount+48] = (unsigned char) (value);
	}
	else if (Target ==Form1->UnitSpaceProv) {
		unit.Data[EntryID*unit.offcount+49] = (unsigned char) (value);
	}
	else if (Target ==Form1->UnitScoreBuild) {
		unit.Data[EntryID*unit.offcount+50] = (unsigned short) (value);
	}
	else if (Target ==Form1->UnitScoreDestroy) {
		unit.Data[EntryID*unit.offcount+51] = (unsigned short) (value);
	}
	else if (Target ==Form1->UnitMinerals) {
		unit.Data[EntryID*unit.offcount+41] = (unsigned short) (value);
	}
	else if (Target ==Form1->UnitVespene) {
		unit.Data[EntryID*unit.offcount+42] = (unsigned short) (value);
	}
	else if (Target ==Form1->UnitTime) {
		unit.Data[EntryID*unit.offcount+43] = (unsigned short) (value*15);
	}
	else if (Target ==Form1->UnitIsBW) {
		unit.Data[EntryID*unit.offcount+53] = bool (value);
	}
        else if (Target ==Form1->UnitSupProv05) {
                     unit.Data[EntryID*unit.offcount+46] = (unsigned char) (Form1->UnitSupProv->Tag*2+int(Form1->UnitSupProv05->Checked));
        }
        else if (Target ==Form1->UnitSupReq05) {
                     unit.Data[EntryID*unit.offcount+47] = (unsigned char) (Form1->UnitSupReq->Tag*2+int(Form1->UnitSupReq05->Checked));
        }
        else if (Target ==Form1->UnitSupProv) {
                     Form1->UnitSupProv->Tag = value;
                     unit.Data[EntryID*unit.offcount+46] = (unsigned char) (Form1->UnitSupProv->Text.ToInt()*2+int(Form1->UnitSupProv05->Checked));
        }
        else if (Target ==Form1->UnitSupReq) {
                     Form1->UnitSupReq->Tag = value;
                     unit.Data[EntryID*unit.offcount+47] = (unsigned char) (Form1->UnitSupReq->Text.ToInt()*2+int(Form1->UnitSupReq05->Checked));
        }

        else if (Target ==Form1->UnitSEGroupProtoss) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+45])|=0x4;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+45])&=~0x4;
              }
        }
        else if (Target ==Form1->UnitSEGroupTerran) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+45])|=0x2;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+45])&=~0x2;
              }
        }
        else if (Target ==Form1->UnitSEGroupZerg) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+45])|=0x1;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+45])&=~0x1;
              }
        }
}
//#############ADVANCED################
else if (Form1->GetSubPage() == 1) {
        if (Target ==Form1->UnitAdvProd) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x80000000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x80000000;
              }
        }
        else if (Target ==Form1->UnitAdvMech) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x40000000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x40000000;
              }
        }
        else if (Target ==Form1->UnitAdvInv) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x20000000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x20000000;
              }
        }
        else if (Target ==Form1->UnitAdvAttackRange) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x10000000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x10000000;
              }
        }
        else if (Target ==Form1->UnitAdvAttackSight) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x8000000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x8000000;
              }
        }
        else if (Target ==Form1->UnitAdvOverlayLarge) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x4000000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x4000000;
              }
        }
        else if (Target ==Form1->UnitAdvOverlayMed) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x2000000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x2000000;
              }
        }
        else if (Target ==Form1->UnitAdvIgnoreSupply) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x1000000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x1000000;
              }
        }
        else if (Target ==Form1->UnitAdvPickup) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x800000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x800000;
              }
        }
        else if (Target ==Form1->UnitAdvPermaCloak) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x400000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x400000;
              }
        }
        else if (Target ==Form1->UnitAdvSpellcaster) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x200000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x200000;
              }
        }
        else if (Target ==Form1->UnitAdvBurrow) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x100000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x100000;
              }
        }
        else if (Target ==Form1->UnitAdvPsi) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x80000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x80000;
              }
        }
        else if (Target ==Form1->UnitAdvUnused) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x40000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x40000;
              }
        }
        else if (Target ==Form1->UnitAdvCreep) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x20000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x20000;
              }
        }
        else if (Target ==Form1->UnitAdvOrg) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x10000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x10000;
              }
        }
        else if (Target ==Form1->UnitAdvDetect) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x8000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x8000;
              }
        }
        else if (Target ==Form1->UnitAdvRobo) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x4000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x4000;
              }
        }
        else if (Target ==Form1->UnitAdvResContainer) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x2000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x2000;
              }
        }
        else if (Target ==Form1->UnitAdvResDepot) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x1000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x1000;
              }
        }
        else if (Target ==Form1->UnitAdvSingle) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x800;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x800;
              }
        }
        else if (Target ==Form1->UnitAdvTwoInEgg) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x400;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x400;
              }
        }
        else if (Target ==Form1->UnitAdvCloak) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x200;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x200;
              }
        }
        else if (Target ==Form1->UnitAdvAnimIdle) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x100;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x100;
              }
        }
        else if (Target ==Form1->UnitAdvReg) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x80;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x80;
              }
        }
        else if (Target ==Form1->UnitAdvHero) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x40;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x40;
              }
        }
        else if (Target ==Form1->UnitAdvFlyBuilding) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x20;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x20;
              }
        }
        else if (Target ==Form1->UnitAdvSub) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x10;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x10;
              }
        }
        else if (Target ==Form1->UnitAdvWorker) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x8;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x8;
              }
        }
        else if (Target ==Form1->UnitAdvFlyer) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x4;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x4;
              }
        }
        else if (Target ==Form1->UnitAdvAddon) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x2;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x2;
              }
        }
        else if (Target ==Form1->UnitAdvBuilding) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+22])|=0x1;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+22])&=~0x1;
              }
        }
//OLD MOVEMENT FLAGS        
        
        else if (Target ==Form1->UnitMov80) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+10])|=0x80;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+10])&=~0x80;
              }
        }
        else if (Target ==Form1->UnitMov40) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+10])|=0x40;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+10])&=~0x40;
              }
        }
        else if (Target ==Form1->UnitMov20) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+10])|=0x20;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+10])&=~0x20;
              }
        }
        else if (Target ==Form1->UnitMov10) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+10])|=0x10;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+10])&=~0x10;
              }
        }
        else if (Target ==Form1->UnitMov08) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+10])|=0x8;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+10])&=~0x8;
              }
        }
        else if (Target ==Form1->UnitMov04) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+10])|=0x4;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+10])&=~0x4;
              }
        }
        else if (Target ==Form1->UnitMov02) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+10])|=0x2;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+10])&=~0x2;
              }
        }
        else if (Target ==Form1->UnitMov01) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+10])|=0x1;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+10])&=~0x1;
              }
        }

        else if (Target ==Form1->UnitUnk1) {
	     unit.Data[EntryID*unit.offcount+44] = (unsigned short) (value);
	}
        else if (Target ==Form1->UnitSub1) {
		unit.Data[EntryID*unit.offcount+1] = (unsigned short) (value);
	}
	else if (Target ==Form1->UnitSub2) {
		unit.Data[EntryID*unit.offcount+2] = (unsigned short) (value);
	}
}
//#############SOUND################
else if (Form1->GetSubPage() == 2) {
        if (Target ==Form1->UnitSndReady) {
		unit.Data[EntryID*unit.offcount+29] = (unsigned short) (value);
                if ((unsigned short) (value) <= Form1->UnitSndReadyList->Items->Count) {
                   Form1->UnitSndReadyList->ItemIndex = value-1;
                }
                else if ((unsigned short) (value) > Form1->UnitSndReadyList->Items->Count) {
                   Form1->UnitSndReadyList->ItemIndex = -1;
                }
	}
        else if (Target == Form1->UnitSndReadyList) {
		unit.Data[EntryID*unit.offcount+29] = (unsigned short) (value+1);
                Form1->UnitSndReady->Text = value+1;
        }

	else if (Target ==Form1->UnitSndWhatStart) {
		unit.Data[EntryID*unit.offcount+30] = (unsigned short) (value);
                if ((unsigned short) (value) <= Form1->UnitSndWhatStartList->Items->Count) {
                   Form1->UnitSndWhatStartList->ItemIndex = value-1;
                }
                else if ((unsigned short) (value) > Form1->UnitSndWhatStartList->Items->Count) {
                   Form1->UnitSndWhatStartList->ItemIndex = -1;
                }
	}
        else if (Target == Form1->UnitSndWhatStartList) {
		unit.Data[EntryID*unit.offcount+30] = (unsigned short) (value+1);
                Form1->UnitSndWhatStart->Text = value+1;
	}

 	else if (Target ==Form1->UnitSndWhatEnd) {
		unit.Data[EntryID*unit.offcount+31] = (unsigned short) (value);
                if ((unsigned short) (value) <= Form1->UnitSndWhatEndList->Items->Count) {
                   Form1->UnitSndWhatEndList->ItemIndex = value-1;
                }
                else if ((unsigned short) (value) > Form1->UnitSndWhatEndList->Items->Count) {
                   Form1->UnitSndWhatEndList->ItemIndex = -1;
                }
	}
        else if (Target == Form1->UnitSndWhatEndList) {
		unit.Data[EntryID*unit.offcount+31] = (unsigned short) (value+1);
                Form1->UnitSndWhatEnd->Text = value+1;
	}

	else if (Target ==Form1->UnitSndAnnStart) {
		unit.Data[EntryID*unit.offcount+32] = (unsigned short) (value);
                if ((unsigned short) (value) <= Form1->UnitSndAnnStartList->Items->Count) {
                   Form1->UnitSndAnnStartList->ItemIndex = value-1;
                }
                else if ((unsigned short) (value) > Form1->UnitSndAnnStartList->Items->Count) {
                   Form1->UnitSndAnnStartList->ItemIndex = -1;
                }
	}
        else if (Target == Form1->UnitSndAnnStartList) {
		unit.Data[EntryID*unit.offcount+32] = (unsigned short) (value+1);
                Form1->UnitSndAnnStart->Text = value+1;
	}

	else if (Target ==Form1->UnitSndAnnEnd) {
		unit.Data[EntryID*unit.offcount+33] = (unsigned short) (value);
                if ((unsigned short) (value) <= Form1->UnitSndAnnEndList->Items->Count) {
                   Form1->UnitSndAnnEndList->ItemIndex = value-1;
                }
                else if ((unsigned short) (value) > Form1->UnitSndAnnEndList->Items->Count) {
                   Form1->UnitSndAnnEndList->ItemIndex = -1;
                }
	}
        else if (Target == Form1->UnitSndAnnEndList) {
		unit.Data[EntryID*unit.offcount+33] = (unsigned short) (value+1);
                Form1->UnitSndAnnEnd->Text = value+1;
       	}

	else if (Target ==Form1->UnitSndYesStart) {
		unit.Data[EntryID*unit.offcount+34] = (unsigned short) (value);
                if ((unsigned short) (value) <= Form1->UnitSndYesStartList->Items->Count) {
                   Form1->UnitSndYesStartList->ItemIndex = value-1;
                }
                else if ((unsigned short) (value) > Form1->UnitSndYesStartList->Items->Count) {
                   Form1->UnitSndYesStartList->ItemIndex = -1;
                }
	}
        else if (Target == Form1->UnitSndYesStartList) {
		unit.Data[EntryID*unit.offcount+34] = (unsigned short) (value+1);
                Form1->UnitSndYesStart->Text = value+1;
 	}

	else if (Target ==Form1->UnitSndYesEnd) {
		unit.Data[EntryID*unit.offcount+35] = (unsigned short) (value);
                if ((unsigned short) (value) <= Form1->UnitSndYesEndList->Items->Count) {
                   Form1->UnitSndYesEndList->ItemIndex = value-1;
                }
                else if ((unsigned short) (value) > Form1->UnitSndYesEndList->Items->Count) {
                   Form1->UnitSndYesEndList->ItemIndex = -1;
                }
	}
        else if (Target == Form1->UnitSndYesEndList) {
		unit.Data[EntryID*unit.offcount+35] = (unsigned short) (value+1);
                Form1->UnitSndYesEnd->Text = value+1;
     	}
}
//###############GRAPHICS################
else if (Form1->GetSubPage() == 3) {
	if (Target ==Form1->UnitGfx) {
		unit.Data[EntryID*unit.offcount] = (unsigned char) (value);
	}
	else if (Target ==Form1->UnitConst) {
		unit.Data[EntryID*unit.offcount+4] = (unsigned long) (value);
	}
	else if (Target ==Form1->UnitInfest) {
		unit.Data[EntryID*unit.offcount+3] = (unsigned short) (value);
	}
	else if (Target ==Form1->UnitPort) {
             if (Form1->UnitPort->ItemIndex == 220) {
		unit.Data[EntryID*unit.offcount+40] = 0xFFFF;
             }
             else {
		unit.Data[EntryID*unit.offcount+40] = (unsigned short) (value);
             }
	}
	else if (Target ==Form1->UnitAnimLevel) {
		unit.Data[EntryID*unit.offcount+9] = (unsigned char) (value);
	}
        else if (Target ==Form1->UnitDirection) {
                unit.Data[EntryID*unit.offcount+5] = (unsigned char) (value);
        }
        if ((Target == Form1->UnitAddPosY || Target == Form1->UnitAddPosX) && Form1->UnitAddPosX->Text.IsEmpty() == false && Form1->UnitAddPosY->Text.IsEmpty() == false) {
             if (Target == Form1->UnitAddPosY) {
                unit.Data[EntryID*unit.offcount+37] = (unsigned int) ((value<<16) + (unsigned short) (Form1->UnitAddPosX->Text.ToInt()));
             }
             else if (Target == Form1->UnitAddPosX) {
                AddonY = long (Form1->UnitAddPosY->Text.ToInt());
                unit.Data[EntryID*unit.offcount+37] = (AddonY<<16) + (unsigned short) value;
             }
        }

        if ((Target == Form1->UnitDimUp || Target == Form1->UnitDimLeft) && Form1->UnitDimLeft->Text.IsEmpty() == false && Form1->UnitDimUp->Text.IsEmpty() == false) {
		if (Target == Form1->UnitDimUp) {
			unit.Data[EntryID*unit.offcount+38] = (value<<16) + (unsigned short) (Form1->UnitDimLeft->Text.ToInt());
		}
		else if (Target == Form1->UnitDimLeft) {
			DimSizeUp = long (Form1->UnitDimUp->Text.ToInt());
			unit.Data[EntryID*unit.offcount+38] = (DimSizeUp<<16) + (unsigned short) value;
		}
        }
	if ((Target == Form1->UnitDimDown || Target == Form1->UnitDimRight) && Form1->UnitDimRight->Text.IsEmpty() == false && Form1->UnitDimDown->Text.IsEmpty() == false) {
		if (Target == Form1->UnitDimDown) {
			unit.Data[EntryID*unit.offcount+39] = (unsigned int) ((value<<16) + (unsigned short) (Form1->UnitDimRight->Text.ToInt()));
		}
		else if (Target == Form1->UnitDimRight) {
			DimSizeDown = long (Form1->UnitDimDown->Text.ToInt());
			unit.Data[EntryID*unit.offcount+39] = (unsigned int) ((DimSizeDown<<16) + (unsigned short) value);
		}
	}
        if (OptionsBox->ShowUnitSizePreview->Checked == true) {
                DrawUnitSize(unit.Data[EntryID*unit.offcount+38],unit.Data[EntryID*unit.offcount+39]);
        }
}
//#############STAREDIT################
else if (Form1->GetSubPage() == 4) {
	if (Target == Form1->UnitMapString) {
	     unit.Data[EntryID*unit.offcount+52] = (unsigned short) (value);
	}
        else if (Target == Form1->UnitRank) {
             unit.Data[EntryID*unit.offcount+11] = (unsigned char) (value);
             if (value < (unsigned int) Form1->UnitSublabel->Items->Count) {
                Form1->UnitSublabel->ItemIndex = (unsigned char) (value);
             }
             else if (value >= (unsigned int) Form1->UnitSublabel->Items->Count) {
                Form1->UnitSublabel->ItemIndex = -1;
             }
        }
        else if (Target == Form1->UnitSublabel) {
             Form1->UnitRank->Text = (unsigned char) (value);
        }

        else if (Target ==Form1->UnitSEGroupNeutral) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+45])|=0x80;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+45])&=~0x80;
              }
        }
        else if (Target ==Form1->UnitSEGroupInd) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+45])|=0x40;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+45])&=~0x40;
              }
        }
        else if (Target ==Form1->UnitSEGroupFactory) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+45])|=0x20;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+45])&=~0x20;
              }
        }
        else if (Target ==Form1->UnitSEGroupBuilding) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+45])|=0x10;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+45])&=~0x10;
              }
        }
        else if (Target ==Form1->UnitSEGroupMen) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+45])|=0x8;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+45])&=~0x8;
              }
        }


        else if (Target ==Form1->UnitSE0x8000) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+54])|=0x8000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+54])&=~0x8000;
              }
        }
        else if (Target ==Form1->UnitSE0x4000) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+54])|=0x4000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+54])&=~0x4000;
              }
        }
        else if (Target ==Form1->UnitSE0x2000) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+54])|=0x2000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+54])&=~0x2000;
              }
        }
        else if (Target ==Form1->UnitSE0x1000) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+54])|=0x1000;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+54])&=~0x1000;
              }
        }
        else if (Target ==Form1->UnitSE0x800) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+54])|=0x800;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+54])&=~0x800;
              }
        }
        else if (Target ==Form1->UnitSE0x400) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+54])|=0x400;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+54])&=~0x400;
              }
        }
        else if (Target ==Form1->UnitSEBW) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+54])|=0x200;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+54])&=~0x200;
              }
        }
        else if (Target ==Form1->UnitSELoc) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+54])|=0x100;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+54])&=~0x100;
              }
        }
        else if (Target ==Form1->UnitSEUnitHero) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+54])|=0x80;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+54])&=~0x80;
              }
        }
        else if (Target ==Form1->UnitSENonLoc) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+54])|=0x40;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+54])&=~0x40;
              }
        }
        else if (Target ==Form1->UnitSEDoodadState) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+54])|=0x20;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+54])&=~0x20;
              }
        }
        else if (Target ==Form1->UnitSEAllRaces) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+54])|=0x10;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+54])&=~0x10;
              }
        }
        else if (Target ==Form1->UnitSEPlayerSet) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+54])|=0x8;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+54])&=~0x8;
              }
        }
        else if (Target ==Form1->UnitSEBriefing) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+54])|=0x4;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+54])&=~0x4;
              }
        }
        else if (Target ==Form1->UnitSEListing) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+54])|=0x2;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+54])&=~0x2;
              }
        }
        else if (Target ==Form1->UnitSENonNeutral) {
              if (value == 1) {
                 (unit.Data[EntryID*unit.offcount+54])|=0x1;
              }
              else {
                 (unit.Data[EntryID*unit.offcount+54])&=~0x1;
              }
        }

       	if (Form1->UnitSEPlaceWidth->Text.IsEmpty() == false && Form1->UnitSEPlaceHeight->Text.IsEmpty() == false) {
		if (Target == Form1->UnitSEPlaceHeight) {
			unit.Data[EntryID*unit.offcount+36] = (unsigned int) ((value<<16) + (unsigned short) (Form1->UnitSEPlaceWidth->Text.ToInt()));
		}
		else if (Target == Form1->UnitSEPlaceWidth) {
			SEPlaceHeight = long (Form1->UnitSEPlaceHeight->Text.ToInt());
			unit.Data[EntryID*unit.offcount+36] = (SEPlaceHeight<<16) + (unsigned short) value;
		}
	}
}
//#############AI ACTIONS################
else if (Form1->GetSubPage() == 5) {
	if (Target == Form1->UnitAICompIdle) {
		unit.Data[EntryID*unit.offcount+12] = (unsigned char) (value);
	}
	else if (Target == Form1->UnitAIHumanIdle) {
		unit.Data[EntryID*unit.offcount+13] = (unsigned char) (value);
	}
	else if (Target == Form1->UnitAIReturn) {
		unit.Data[EntryID*unit.offcount+14] = (unsigned char) (value);
	}
	else if (Target == Form1->UnitAIAttackUnit) {
		unit.Data[EntryID*unit.offcount+15] = (unsigned char) (value);
	}
	else if (Target == Form1->UnitAIAttackMove) {
		unit.Data[EntryID*unit.offcount+16] = (unsigned char) (value);
	}
        else if (Target == Form1->UnitAIRightClick) {
                unit.Data[EntryID*unit.offcount+28] = (unsigned char) (value);
        }
        else if (Target == Form1->UnitAIInternal) {
                unit.Data[EntryID*unit.offcount+21] = (unsigned char) (value);
        }
}
};

void ChangeWeaponData (int value,int EntryID,TObject* Target) {
	if (Target == Form1->WeapLabel) {
	     weapon.Data[EntryID*weapon.offcount] =  (unsigned short) (value);
	}
        else if (Target == Form1->WeapGraphics) {
           weapon.Data[EntryID*weapon.offcount+1] = (unsigned long) (value);
           if (value < Form1->WeapGraphicsList->Items->Count) {
              Form1->WeapGraphicsList->ItemIndex = (unsigned long) (value);
           }
           else if (value >= Form1->WeapGraphicsList->Items->Count) {
              Form1->WeapGraphicsList->ItemIndex = -1;
           }
        }
	else if (Target == Form1->WeapGraphicsList) {
             Form1->WeapGraphics->Text = (unsigned long) (value);
	}
	else if (Target == Form1->WeapUnused) {
	     weapon.Data[EntryID*weapon.offcount+2] = (unsigned char) (value);
	}
	else if (Target == Form1->WeapRangeMin) {
	     weapon.Data[EntryID*weapon.offcount+4] = (unsigned long) (value*16);
	}
	else if (Target == Form1->WeapRangeMax) {
	     weapon.Data[EntryID*weapon.offcount+5] = (unsigned long) (value*16);
	}
	else if (Target == Form1->WeapUpgrade) {
	     weapon.Data[EntryID*weapon.offcount+6] = (unsigned char) (value);
	}
	else if (Target == Form1->WeapType) {
	     weapon.Data[EntryID*weapon.offcount+7] = (unsigned char) (value);
	}
	else if (Target == Form1->WeapBehaviour) {
	     weapon.Data[EntryID*weapon.offcount+8] = (unsigned char) (value);
	}
	else if (Target == Form1->WeapMissile) {
	     switch (value) {
	       case 0:
		    weapon.Data[EntryID*weapon.offcount+9] = 0x1E;
		    break;
	       case 1:
		    weapon.Data[EntryID*weapon.offcount+9] = 0x3C;
		    break;
	       case 2:
		    weapon.Data[EntryID*weapon.offcount+9] = 0xFF;
                    break;
             }
	}
        else if (Target == Form1->WeapExplosion) {
	     weapon.Data[EntryID*weapon.offcount+10] = (unsigned char) (value);
        }
        else if (Target == Form1->WeapInner) {
             weapon.Data[EntryID*weapon.offcount+11] = (unsigned short) (value);
        }
	else if (Target == Form1->WeapMed) {
             weapon.Data[EntryID*weapon.offcount+12] = (unsigned short) (value);
        }
        else if (Target == Form1->WeapOuter) {
             weapon.Data[EntryID*weapon.offcount+13] = (unsigned short) (value);
        }
        else if (Target == Form1->WeapAmount) {
             weapon.Data[EntryID*weapon.offcount+14] = (unsigned short) (value);
        }
        else if (Target == Form1->WeapBonus) {
             weapon.Data[EntryID*weapon.offcount+15] = (unsigned short) (value);
        }
	else if (Target == Form1->WeapCooldown) {
             weapon.Data[EntryID*weapon.offcount+16] = (unsigned char) (value);
        }
        else if (Target == Form1->WeapFactor) {
             weapon.Data[EntryID*weapon.offcount+17] = (unsigned char) (value);
        }
        else if (Target == Form1->WeapError) {
             weapon.Data[EntryID*weapon.offcount+22] = (unsigned short) (value);
        }
        else if (Target == Form1->WeapIcon) {
           weapon.Data[EntryID*weapon.offcount+23] = (unsigned short) (value);
           if (value < Form1->WeapIconList->Items->Count) {
              Form1->WeapIconList->ItemIndex = (unsigned short) (value);
              DrawIcon(value,Form1->WeapIconPrev);
           }
           else if (value >= Form1->WeapIconList->Items->Count) {
              Form1->WeapIconList->ItemIndex = -1;
              DrawIcon(-1,Form1->WeapIconPrev);
           }
        }
	else if (Target == Form1->WeapIconList) {
             Form1->WeapIcon->Text = (unsigned short) (value);
	}

        else if (Target == Form1->WeapAttackAngle) {
		weapon.Data[EntryID*weapon.offcount+18] = (unsigned short) (value);
        }
        else if (Target == Form1->WeapLaunchSpin) {
		weapon.Data[EntryID*weapon.offcount+19] = (unsigned short) (value);
        }
        else if (Target == Form1->WeapForwardOffset) {
		weapon.Data[EntryID*weapon.offcount+20] = (unsigned short) (value);
        }
        else if (Target == Form1->WeapUpwardOffset) {
		weapon.Data[EntryID*weapon.offcount+21] = (unsigned short) (value);
        }

        if (Target ==Form1->WeapTarget100) {
              if (value == 1) {
                 (weapon.Data[EntryID*weapon.offcount+3])|=0x100;
              }
              else {
                 (weapon.Data[EntryID*weapon.offcount+3])&=~0x100;
              }
        }
        else if (Target ==Form1->WeapTarget080) {
              if (value == 1) {
                 (weapon.Data[EntryID*weapon.offcount+3])|=0x80;
              }
              else {
                 (weapon.Data[EntryID*weapon.offcount+3])&=~0x80;
              }
        }
        else if (Target ==Form1->WeapTarget040) {
              if (value == 1) {
                 (weapon.Data[EntryID*weapon.offcount+3])|=0x40;
              }
              else {
                 (weapon.Data[EntryID*weapon.offcount+3])&=~0x40;
              }
        }
        else if (Target ==Form1->WeapTarget020) {
              if (value == 1) {
                 (weapon.Data[EntryID*weapon.offcount+3])|=0x20;
              }
              else {
                 (weapon.Data[EntryID*weapon.offcount+3])&=~0x20;
              }
        }
        else if (Target ==Form1->WeapTarget010) {
              if (value == 1) {
                 (weapon.Data[EntryID*weapon.offcount+3])|=0x10;
              }
              else {
                 (weapon.Data[EntryID*weapon.offcount+3])&=~0x10;
              }
        }
        else if (Target ==Form1->WeapTarget008) {
              if (value == 1) {
                 (weapon.Data[EntryID*weapon.offcount+3])|=0x8;
              }
              else {
                 (weapon.Data[EntryID*weapon.offcount+3])&=~0x8;
              }
        }
        else if (Target ==Form1->WeapTarget004) {
              if (value == 1) {
                 (weapon.Data[EntryID*weapon.offcount+3])|=0x4;
              }
              else {
                 (weapon.Data[EntryID*weapon.offcount+3])&=~0x4;
              }
        }
        else if (Target ==Form1->WeapTarget002) {
              if (value == 1) {
                 (weapon.Data[EntryID*weapon.offcount+3])|=0x2;
              }
              else {
                 (weapon.Data[EntryID*weapon.offcount+3])&=~0x2;
              }
        }
        else if (Target ==Form1->WeapTarget001) {
              if (value == 1) {
                 (weapon.Data[EntryID*weapon.offcount+3])|=0x1;
              }
              else {
                 (weapon.Data[EntryID*weapon.offcount+3])&=~0x1;
              }
        }
};

void ChangeFlingyData (int value,int EntryID,TObject* Target) {
      if (Target == Form1->FlingySprite) {
         flingy.Data[EntryID*flingy.offcount] = (unsigned short) (value);
         if ((unsigned short) (value) < Form1->FlingySpriteList->Items->Count) {
             Form1->FlingySpriteList->ItemIndex = value;
         }
         else if ((unsigned short) (value) >= Form1->FlingySpriteList->Items->Count) {
             Form1->FlingySpriteList->ItemIndex = -1;
         }
      }
      else if (Target == Form1->FlingySpriteList) {
           Form1->FlingySprite->Text = (unsigned short) (value);
      }
      else if (Target == Form1->FlingySpeed) {
         flingy.Data[EntryID*flingy.offcount+1] = (unsigned long) (value);
      }
      else if (Target == Form1->FlingyAcceleration) {
	 flingy.Data[EntryID*flingy.offcount+2] = (unsigned short) (value);
      }
      else if (Target == Form1->FlingyHaltDist) {
         flingy.Data[EntryID*flingy.offcount+3] = (unsigned long) (value);
      }
      else if (Target == Form1->FlingyTurnRad) {
	 flingy.Data[EntryID*flingy.offcount+4] = (unsigned char) (value);
      }
      else if (Target == Form1->FlingyUnk) {
         flingy.Data[EntryID*flingy.offcount+5] = (unsigned char) (value);
      }
      else if (Target == Form1->FlingyMoveControl) {
         flingy.Data[EntryID*flingy.offcount+6] = (unsigned char) (value);
      }
};

void ChangeSpriteData (int value,int EntryID,TObject* Target) {
        if (Target == Form1->SpriteFile) {
           sprite.Data[EntryID*sprite.offcount] = (unsigned short) (value);
           if (value < Form1->SpriteImageList->Items->Count) {
               Form1->SpriteImageList->ItemIndex = value;
           }
           else if (value >= Form1->SpriteImageList->Items->Count) {
               Form1->SpriteImageList->ItemIndex = -1;
           }
   	}
        else if (Target == Form1->SpriteImageList) {
           Form1->SpriteFile->Text = value;
        }
        else if (Target == Form1->SpriteUnk1) {
           sprite.Data[EntryID*sprite.offcount+2] = bool (value);
        }
	else if (Target == Form1->SpriteVisible) {
           sprite.Data[EntryID*sprite.offcount+3] = bool (value);
        }
        if (EntryID >= 130) {
           if (Target == Form1->SpriteHPBar) {
              sprite.Data[(EntryID)*sprite.offcount+1] = (unsigned char) (value);
	   }
           else if (Target == Form1->SpriteCircleSize) {
              Form1->SpriteSelCircle->Text = value;
           }
           else if (Target == Form1->SpriteSelCircle) {
              sprite.Data[(EntryID)*sprite.offcount+4] = (unsigned char) (value);
           }
           else if (Target == Form1->SpriteCircleOff) {
              sprite.Data[(EntryID)*sprite.offcount+5] = (unsigned char) (value);
           }
	}
};

void ChangeImageData (int value,int EntryID,TObject* Target) {
      if (Target == Form1->ImgGRP) {
         image.Data[EntryID*image.offcount] = (unsigned long) (value);
         if (value > Form1->ImgGRPTBL->Items->Count) {
            Form1->ImgGRPTBL->ItemIndex = -1;
         }
         else if (value <= Form1->ImgGRPTBL->Items->Count) {
            Form1->ImgGRPTBL->ItemIndex = value-1;
         }
      }
      else if (Target == Form1->ImgGRPTBL) {
         Form1->ImgGRP->Text = value+1;
      }
      else if (Target == Form1->ImgIscriptID) {
           image.Data[EntryID*image.offcount+7] = (unsigned long) (value);
           if (value < Form1->ImgIscriptIDList->Items->Count) {
               Form1->ImgIscriptIDList->ItemIndex = value;
           }
           else if (value >= Form1->ImgIscriptIDList->Items->Count) {
               Form1->ImgIscriptIDList->ItemIndex = -1;
           }
      }
      else if (Target == Form1->ImgIscriptIDList) {
           image.Data[EntryID*image.offcount+7] = (unsigned long) (value);
           Form1->ImgIscriptID->Text = value;
      }
      else if (Target == Form1->ImgGfxTurns) {
         image.Data[EntryID*image.offcount+1] = bool (value);
      }
      else if (Target == Form1->ImgClickable) {
         image.Data[EntryID*image.offcount+2] = bool (value);
      }
      else if (Target == Form1->ImgUseIscript) {
         image.Data[EntryID*image.offcount+3] = bool (value);
      }
      else if (Target == Form1->ImgDrawCloaked) {
	 image.Data[EntryID*image.offcount+4] = bool (value);
      }
      else if (Target == Form1->ImgPalette) {
         switch (Form1->ImgPalette->ItemIndex) {
               case 0:
                    image.Data[EntryID*image.offcount+5]=0;
                    break;
               case 1:
                    image.Data[EntryID*image.offcount+5]=1;
                    break;
               case 2:
                    image.Data[EntryID*image.offcount+5]=3;
                    break;
	       case 3:
                    image.Data[EntryID*image.offcount+5]=8;
		    break;
               case 4:
                    image.Data[EntryID*image.offcount+5]=9;
                    break;
               case 5:
		    image.Data[EntryID*image.offcount+5]=10;
                    break;
               case 6:
                    image.Data[EntryID*image.offcount+5]=17;
                    break;
        }
      }
      else if (Target == Form1->ImgPalSpec) {
           image.Data[EntryID*image.offcount+6] = (unsigned char) (value);
      }
      else if (Target == Form1->ImgShield) {
           image.Data[EntryID*image.offcount+8] = (unsigned long) (value);
           Form1->ImgShieldTBL->ItemIndex = value-1;
           switch (value) {
               case 0:
                    Form1->ImgShieldOL->ItemIndex=0;
                    break;
               case 133:
                    Form1->ImgShieldOL->ItemIndex=1;
                    break;
               case 2:
                    Form1->ImgShieldOL->ItemIndex=2;
		    break;
               case 184:
                    Form1->ImgShieldOL->ItemIndex=3;
                    break;
               default:
                    Form1->ImgShieldOL->ItemIndex=-1;
                    break;
           }
      }
      else if (Target == Form1->ImgShieldOL) {
        switch (Form1->ImgShieldOL->ItemIndex) {
               case 0:
                    Form1->ImgShield->Text=0;
                    break;
               case 1:
                    Form1->ImgShield->Text=133;
                    break;
               case 2:
                    Form1->ImgShield->Text=2;
		    break;
               case 3:
		    Form1->ImgShield->Text=184;
                    break;
        }
      }
      else if (Target == Form1->ImgShieldTBL) {
	   image.Data[EntryID*image.offcount+8] = (unsigned long) (value+1);
           Form1->ImgShield->Text = value+1;
      }

      else if (Target == Form1->ImgOL1) {
	   image.Data[EntryID*image.offcount+9] = (unsigned long) (value);
           Form1->ImgOL1TBL->ItemIndex = value-1;
      }
      else if (Target == Form1->ImgOL1TBL) {
	   image.Data[EntryID*image.offcount+9] = (unsigned long) (value+1);
           Form1->ImgOL1->Text = value+1;
      }
      else if (Target == Form1->ImgOL2) {
           image.Data[EntryID*image.offcount+10] = (unsigned long) (value);
           Form1->ImgOL2TBL->ItemIndex = value-1;
      }
      else if (Target == Form1->ImgOL2TBL) {
	   image.Data[EntryID*image.offcount+10] = (unsigned long) (value+1);
           Form1->ImgOL2->Text = value+1;
      }
      else if (Target == Form1->ImgOL3) {
	   image.Data[EntryID*image.offcount+11] = (unsigned long) (value);
           Form1->ImgOL3TBL->ItemIndex = value-1;
      }
      else if (Target == Form1->ImgOL3TBL) {
	   image.Data[EntryID*image.offcount+11] = (unsigned long) (value+1);
           Form1->ImgOL3->Text = value+1;
      }
      else if (Target == Form1->ImgOL4) {
           image.Data[EntryID*image.offcount+12] = (unsigned long) (value);
           Form1->ImgOL4TBL->ItemIndex = value-1;
      }
      else if (Target == Form1->ImgOL4TBL) {
	   image.Data[EntryID*image.offcount+12] = (unsigned long) (value+1);
           Form1->ImgOL4->Text = value+1;
      }
      else if (Target == Form1->ImgOL5) {
           image.Data[EntryID*image.offcount+13] = (unsigned long) (value);
           Form1->ImgOL5TBL->ItemIndex = value-1;
      }
      else if (Target == Form1->ImgOL5TBL) {
	   image.Data[EntryID*image.offcount+13] = (unsigned long) (value+1);
           Form1->ImgOL5->Text = value+1;
      }
};

void ChangeUpgradeData (int value,int EntryID,TObject* Target) {
	if (Target == Form1->UpgMinerals) {
		upgrade.Data[EntryID*upgrade.offcount] = (unsigned short) (value);
	}
	else if (Target == Form1->UpgFactorMinerals) {
		upgrade.Data[EntryID*upgrade.offcount+1] = (unsigned short) (value);
	}
	else if (Target == Form1->UpgVespene) {
		upgrade.Data[EntryID*upgrade.offcount+2] = (unsigned short) (value);
	}
	else if (Target == Form1->UpgFactorVespene) {
		upgrade.Data[EntryID*upgrade.offcount+3] = (unsigned short) (value);
	}
	else if (Target == Form1->UpgTime) {
		upgrade.Data[EntryID*upgrade.offcount+4] = (unsigned short) (value*15);
	}
	else if (Target == Form1->UpgFactorTime) {
		upgrade.Data[EntryID*upgrade.offcount+5] = (unsigned short) (value*15);
	}
	else if (Target == Form1->UpgUnk) {
		upgrade.Data[EntryID*upgrade.offcount+6] = (unsigned short) (value);
	}
        else if (Target == Form1->UpgIcon) {
		upgrade.Data[EntryID*upgrade.offcount+7] = (unsigned short) (value);
                if ((unsigned short) (value) < Form1->UpgIconList->Items->Count) {
                   Form1->UpgIconList->ItemIndex = (unsigned short) (value);
                   DrawIcon(value,Form1->UpgIconPrev);
                }
                else if (value >= Form1->UpgIconList->Items->Count) {
                   Form1->UpgIconList->ItemIndex = -1;
                   DrawIcon(-1,Form1->UpgIconPrev);
                }
        }
	else if (Target == Form1->UpgIconList) {
                Form1->UpgIcon->Text = (unsigned short) (value);
	}
        else if (Target == Form1->UpgLabel) {
		upgrade.Data[EntryID*upgrade.offcount+8] = (unsigned short) (value);
                if (value <= Form1->UpgLabelList->Items->Count) {
                   Form1->UpgLabelList->ItemIndex = value;
                }
                else  {
                   Form1->UpgLabelList->ItemIndex = -1;
                }
	}
        else if (Target ==  Form1->UpgLabelList) {
                Form1->UpgLabel->Text = (unsigned short) (value);
        }

        
	else if (Target == Form1->UpgRace) {
		upgrade.Data[EntryID*upgrade.offcount+9] = (unsigned char) (value);
	}
	else if (Target == Form1->UpgRepeats) {
		upgrade.Data[EntryID*upgrade.offcount+10] = (unsigned char) (value);
	}
	else if (Target == Form1->UpgIsBW) {
		upgrade.Data[EntryID*upgrade.offcount+11] = bool (value);
	}
};

void ChangeTechData (int value,int EntryID,TObject* Target) {
	if (Target == Form1->TechMinerals) {
		tech.Data[EntryID*tech.offcount] = (unsigned short) (value);
	}
	else if (Target == Form1->TechVespene) {
		tech.Data[EntryID*tech.offcount+1] = (unsigned short) (value);
	}
	else if (Target == Form1->TechTime) {
		tech.Data[EntryID*tech.offcount+2] = (unsigned short) (value*15);
	}
	else if (Target == Form1->TechMana) {
		tech.Data[EntryID*tech.offcount+3] = (unsigned short) (value);
	}
	else if (Target == Form1->TechUnk) {
		tech.Data[EntryID*tech.offcount+4] = (unsigned long) (value);
	}
        else if (Target == Form1->TechIcon) {
		tech.Data[EntryID*tech.offcount+5] = (unsigned short) (value);
                if ((unsigned short) (value) < Form1->TechIconList->Items->Count) {
                   Form1->TechIconList->ItemIndex = (unsigned short) (value);
                   DrawIcon(value,Form1->TechIconPrev);
                }
                else if (value >= Form1->TechIconList->Items->Count) {
                   Form1->TechIconList->ItemIndex = -1;
                   DrawIcon(-1,Form1->TechIconPrev);
                }
        }
        else if (Target == Form1->TechIconList) {
                Form1->TechIcon->Text = (unsigned short) (value);
        }
        
        else if (Target == Form1->TechLabel) {
		tech.Data[EntryID*tech.offcount+6] = (unsigned short) (value);
                if (value <= Form1->TechLabelList->Items->Count) {
                   Form1->TechLabelList->ItemIndex = value;
                }
                else {
                   Form1->TechLabelList->ItemIndex = -1;
                }
	}
        else if (Target == Form1->TechLabelList) {
                Form1->TechLabel->Text = (unsigned short) (value);
	}

	else if (Target == Form1->TechRace) {
		tech.Data[EntryID*tech.offcount+7] = (unsigned char) (value);
	}
	else if (Target == Form1->TechUnused) {
		tech.Data[EntryID*tech.offcount+8] = bool (value);
	}
	else if (Target == Form1->TechBW) {
		tech.Data[EntryID*tech.offcount+9] = bool (value);
	}
};

void ChangeSoundData(int value,int EntryID,TObject* Target) {
      if (Target == Form1->SoundFile) {
	   sound.Data[EntryID*sound.offcount] = (unsigned long) (value);
           if (value <= Form1->SoundTBL->Items->Count) {
               Form1->SoundTBL->ItemIndex = value;
           }
           else
               Form1->SoundTBL->ItemIndex = -1;
      }
      else if (Target == Form1->SoundTBL) {
           Form1->SoundFile->Text = (unsigned long) (value);
      }

      else if (Target == Form1->SoundUnk1) {
	   sound.Data[EntryID*sound.offcount+1] = (unsigned char) (value);
      }
      else if (Target == Form1->SoundUnk2) {
	   sound.Data[EntryID*sound.offcount+2] = (unsigned char) (value);
      }
      else if (Target == Form1->SoundUnk3) {
	   sound.Data[EntryID*sound.offcount+3] = (unsigned short) (value);
      }
      else if (Target == Form1->SoundUnk4) {
	   sound.Data[EntryID*sound.offcount+4] = (unsigned char) (value);
      }
};

void ChangePortData (int value,int EntryID,TObject* Target) {
      if (Target == Form1->PortFile) {
           portrait.Data[EntryID*portrait.offcount] = (unsigned long) (value);
           if (value <= Form1->PortTBL->Items->Count) {
              Form1->PortTBL->ItemIndex = value-1;
           }
           else
              Form1->PortTBL->ItemIndex = -1;
      }
      else if (Target == Form1->PortTBL) {
           Form1->PortFile->Text = (unsigned long) (value+1);
      }
      else if (Target == Form1->PortSMKChange) {
	   portrait.Data[(EntryID*portrait.offcount)+1] = (unsigned char) (value);
      }
      else if (Target == Form1->PortUnk1) {
	   portrait.Data[(EntryID*portrait.offcount)+2] = (unsigned char) (value);
      }
};

void ChangeMapData (int value,int EntryID,TObject* Target) {
	if (Target == Form1->MapFile) {
	   map.Data[EntryID] = (unsigned long) (value);
           if (value <= Form1->MapTBL->Items->Count) {
              Form1->MapTBL->ItemIndex = value;
           }
           else
              Form1->MapTBL->ItemIndex = -1;
	}
        else if (Target == Form1->MapTBL) {
           Form1->MapFile->Text = (unsigned long) (value);
        }
};

void ChangeOrderData (int value,int EntryID,TObject* Target) {
	if (Target == Form1->OrdLabel) {
                order.Data[EntryID*order.offcount] = (unsigned short) (value);
	}
	else if (Target == Form1->OrdWeapTarg) {
	        order.Data[EntryID*order.offcount+1] = bool (value);
	}
	else if (Target == Form1->OrdUnk2) {
		order.Data[EntryID*order.offcount+2] = bool (value);
	}
	else if (Target == Form1->OrdUnk3) {
		order.Data[EntryID*order.offcount+3] = bool (value);
	}
	else if (Target == Form1->OrdUnk4) {
		order.Data[EntryID*order.offcount+4] = bool (value);
	}
	else if (Target == Form1->OrdUnk5) {
		order.Data[EntryID*order.offcount+5] = bool (value);
	}
	else if (Target == Form1->OrdInterupt) {
		order.Data[EntryID*order.offcount+6] = bool (value);
	}
	else if (Target == Form1->OrdUnk7) {
		order.Data[EntryID*order.offcount+7] = bool (value);
	}
	else if (Target == Form1->OrdQueue) {
		order.Data[EntryID*order.offcount+8] = bool (value);
	}
	else if (Target == Form1->OrdUnk9) {
		order.Data[EntryID*order.offcount+9] = bool (value);
	}
	else if (Target == Form1->OrdUnk10) {
		order.Data[EntryID*order.offcount+10] = bool (value);
	}
	else if (Target == Form1->OrdUnk11) {
		order.Data[EntryID*order.offcount+11] = bool (value);
	}
	else if (Target == Form1->OrdUnk12) {
		order.Data[EntryID*order.offcount+12] = bool (value);
	}
	else if (Target == Form1->OrdWeapon) {
		order.Data[EntryID*order.offcount+13] = (unsigned char) (value);
	}
	else if (Target == Form1->OrdTech) {
		order.Data[EntryID*order.offcount+14] = (unsigned char) (value);
	}
	else if (Target == Form1->OrdAnimation) {
		order.Data[EntryID*order.offcount+15] = (unsigned char) (value);
	}
        else if (Target == Form1->OrdHighlight) {
		order.Data[EntryID*order.offcount+16] = (unsigned short) (value);
                if ((unsigned short) (value) < Form1->OrdHighlightList->Items->Count) {
                   Form1->OrdHighlightList->ItemIndex = (unsigned short) (value);
                   DrawIcon(value,Form1->OrdHighlightPrev);
                }
                else if (value >= Form1->OrdHighlightList->Items->Count) {
                   Form1->OrdHighlightList->ItemIndex = -1;
                   DrawIcon(-1,Form1->OrdHighlightPrev);
                }
        }
        else if (Target == Form1->OrdHighlightList) {
                Form1->OrdHighlight->Text = (unsigned short) (value);
        }

	else if (Target == Form1->OrdUnk) {
		order.Data[EntryID*order.offcount+17] = (unsigned short) (value);
	}
	else if (Target == Form1->OrdEmulate) {
		order.Data[EntryID*order.offcount+18] = (unsigned char) (value);
	}
};
//------------------------------------------------------------------------------
/*
############################################################################
Draw the Unit Size Preview in the Graphics tab of the Units Editor.
The reference point is put exactly at the center of the Preview
panel and the Unit Size rectangle is also centered in a similar way.
############################################################################
*/

void DrawUnitSize(int ViewLeft,int ViewRight) {
     int SizeLeft = (unsigned short) (ViewLeft);
     int SizeUp = (unsigned short) (ViewLeft>>16);
     int SizeRight = (unsigned short) (ViewRight);
     int SizeDown = (unsigned short) (ViewRight>>16);
     Form1->SizeRefPoint->Left = (Form1->UnitDimView->Width)/2;
     Form1->SizeRefPoint->Top = (Form1->UnitDimView->Height)/2;

     Form1->SizeRect->Width = SizeLeft+SizeRight;
     Form1->SizeRect->Height = SizeUp+SizeDown;
     Form1->SizeRect->Left = (Form1->UnitDimView->Width)/2;
     Form1->SizeRect->Top = (Form1->UnitDimView->Height)/2;
     if (SizeUp==1 && SizeDown==1) {
          Form1->SizeRect->Top = (Form1->UnitDimView->Height)/2;
     }
     else if (SizeLeft==1 && SizeUp==1) {
          Form1->SizeRect->Left = (Form1->UnitDimView->Width)/2;
     }
     else {
          Form1->SizeRect->Top = (Form1->UnitDimView->Height)/2+1-SizeUp;
          Form1->SizeRect->Left = (Form1->UnitDimView->Width)/2+1-SizeLeft;
     }
};
//------------------------------------------------------------------------------
/*
############################################################################
Draw a specific icon on a specific TImage control. Both are supplied
externally as parameters. There is a black rectangle (FillRect) drawn
prior to the GRP frame itself. This is because of the way transparency
works with TImage components and it was necessary to 'manually' clear
out the background using the said rectangle.
############################################################################
*/
void DrawIcon(int IconIndex,TImage *TargetPreview) {
     unsigned long PalBuffer[256];
     void *GRPBuffer;

     TargetPreview->Visible = true;
     TargetPreview->Canvas->Brush->Color = clBlack;
     TargetPreview->Canvas->FillRect(Rect(0,0,32,32));
     if (IconIndex != -1) {
        if (Form1->GetIconIsMPQImport() == false) {
           GRPBuffer = LoadGrp(Form1->GetIcons().c_str());
        }
        else if (Form1->GetIconIsMPQImport() == true) {
           GRPBuffer = LoadGrp(Form1->GetMPQIcons().c_str());
        }
        LoadPalette(String(Form1->GetProgPath()+"default\\IconPal.pal").c_str(), PalBuffer);

        unsigned long left, top, width, height;
        GetGrpFrameInfo(GRPBuffer, IconIndex, &left, &top, &width, &height);
        DrawGrp(GRPBuffer, TargetPreview->Canvas->Handle, abs(TargetPreview->Width-width)/2, abs(TargetPreview->Height-height)/2, IconIndex, PalBuffer, 0,0);
        DestroyGrp(GRPBuffer);
     }
};
//------------------------------------------------------------------------------
