//---------------------------------------------------------------------------
#pragma hdrstop

#include "LoadList.h"
#include "TBLParser.h"
#include "Data.h"
#include "EntryData.h"
#include "Options.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
/*
############################################################################
Return the Index of an entry in the left-hand listing when given a specific
ID first.
############################################################################
*/

int LookEntry (int TargetID,int TargetPage) {
     String TargetEntryName;
     int ReturnID=NULL;
     for (int i=0;i<GetDATList(TargetPage)->Items->Count;i++) {
         if (GetDATList(TargetPage)->Items->Item[i]->ImageIndex == TargetID) {
            ReturnID = i;
            break;
         }
     }
     return ReturnID;
};
//---------------------------------------------------------------------------
/*
############################################################################
Load the content of a TBL file to a designated interface element
(precisely, to a dropdown list aka a "combobox"). Use an invisible
intermediate TBLBuf TMemo-type interface element, because the
list content of a combobox is of the same type (TStrings) as the content of
a TMemo, and it allows loading (i.e. copying from TBLBuf) of the entire
list instead of adding one item at a time.

Also add a "No Label" entry at the beginning of the list if the TBL
used is stat_txt.tbl.
############################################################################
*/

void LoadTBLList (TComboBox* Target,String TBLType) {
Target->Items->Clear();
Form1->TBLBuf->Clear();

for (int i=0;i<5;i++) {
    if (TBLType == GetTBLType(i)) {
       if (Form1->GetTBLIsMPQImport(i) == false) {
          ReadTBL(Form1->GetTBL(i), Form1->TBLBuf);
       }
       else {
          ReadTBL(Form1->GetMPQTBL(i), Form1->TBLBuf);
       }
       if (i==0) {
          Target->Items->Add("No Label");
       }
       Form1->TBLBuf->Tag = i;
       break;
    }
}
Target->Items->AddStrings(Form1->TBLBuf->Lines);
};
//---------------------------------------------------------------------------
/*
############################################################################
Loads the content of a text file into a dropdown list.
############################################################################
*/

void LoadList (TComboBox* Target,String List) {
     Target->Items->Clear();
     if (FileExists(Form1->GetProgPath()+"Data\\"+List+".txt")) {
        Target->Items->LoadFromFile(Form1->GetProgPath()+"Data\\"+List+".txt");
     }
     else
        MessageDlg(("The following list was not found: "+List+".txt\n DatEdit will continue, but consider reinstalling the program.").c_str(), mtError, TMsgDlgButtons() << mbOK, 0);
};
//---------------------------------------------------------------------------
/*
############################################################################
Load the content of a TBL file into an invisible intermediate TMemo
component, used to store the TBL lines to display them in
another, this time visible, control.

Used in the context of the Sound, Map, Portrait and Images Editors. 
############################################################################
*/
void LoadTBLMemo (TMemo* Target, String TBLType) {
     for (int i=0;i<5;i++) {
         if (TBLType == GetTBLType(i)) {
            ReadTBL(Form1->GetTBL(i),Target);
            break;
         }
     }
};
//---------------------------------------------------------------------------
/*
############################################################################
Load the entry names to a treeview-type list (on the left-hand side
of the application window).

Skip the item labeled "None", since there is no such data entry in the
DAT files. This list item must be there however, because the same listfiles
are also used for the left-hand side listings and the pointer dropdown boxes.
############################################################################
*/

void LoadListID (String ListFile, TTreeView* List) {
     List->ClearSelection();
     List->Items->Clear();
     List->Show();
     List->ShowRoot = false;
     List->ShowLines = false;
     List->ShowButtons = false;

     if (Form1->GetPage() == 0) {
        TIniFile *ConfigIni = new TIniFile(Form1->GetProgPath()+"DatEdit.ini");
        Form1->SetUseCustomNames(ConfigIni->ReadBool("Options","UseCustomNames",false));
        delete ConfigIni;
     }

     if (Form1->GetPage() != 0 || (Form1->GetPage() == 0 && Form1->GetUseCustomNames() == false)) {
        Form1->TextBuf->Lines->LoadFromFile(Form1->GetProgPath()+"Data\\"+ListFile+".txt");
     }
     List->Items->BeginUpdate();
     for (int i=0;i<GetDATObject(Form1->GetPage())->GetDatacount();i++) {
         if (Form1->TextBuf->Lines->Strings[i].Pos("None") != 0) {
	    continue;
         }
         TTreeNode *Dummy;
         if (Form1->GetPage() == 0 && Form1->GetUseCustomNames() == true) {
   	    Dummy = List->Items->AddChild(NULL, LoadName(i));
         }
         else {
            Dummy = List->Items->AddChild(NULL, Form1->TextBuf->Lines->Strings[i]);
         }
         Dummy->ImageIndex = i;
     }
     List->Items->EndUpdate();
};
//---------------------------------------------------------------------------
/*
############################################################################
Loads a list with the DAT entries into the left-side listing, but when the
'Sort by Origin' option is selected.
Also check and remove the empty categories from the tree view.
############################################################################
*/

void LoadListOrigin (String ListFile, TTreeView* List) {
     List->ClearSelection();
     List->Items->Clear();
     List->Show();
     List->ShowRoot = true;
     List->ShowLines = true;
     List->ShowButtons = true;
     if (Form1->GetPage() == 0) {
        TIniFile *ConfigIni = new TIniFile(Form1->GetProgPath()+"DatEdit.ini");
        Form1->SetUseCustomNames(ConfigIni->ReadBool("Options","UseCustomNames",false));
        delete ConfigIni;
     }
     if (Form1->GetPage() != 0 || (Form1->GetPage() == 0 && Form1->GetUseCustomNames() == false)) {
        Form1->TextBuf->Lines->LoadFromFile(Form1->GetProgPath()+"Data\\"+ListFile+".txt");
     }

     List->Items->BeginUpdate();
     HTREEITEM MainNodes[4];
     MainNodes[0] = (List->Items->AddChild(NULL, "Zerg "+GetDataType(Form1->GetPage())))->ItemId;
     MainNodes[1] = (List->Items->AddChild(NULL, "Terran "+GetDataType(Form1->GetPage())))->ItemId;
     MainNodes[2] = (List->Items->AddChild(NULL, "Protoss "+GetDataType(Form1->GetPage())))->ItemId;
     MainNodes[3] = (List->Items->AddChild(NULL, "Other "+GetDataType(Form1->GetPage())))->ItemId;
     for (int i=0;i<4;i++) {
         List->Items->GetNode(MainNodes[i])->ImageIndex = -1;
     }
     if (Form1->GetPage() != 0) {
         for (int i=0;i<(Form1->TextBuf->Lines->Count);i++) {
            if (Form1->TextBuf->Lines->Strings[i].Pos("None") != 0) {
	       continue;
            }
            TTreeNode* Dummy;
            Dummy = List->Items->AddChild(List->Items->GetNode(MainNodes[GetOrigin(i,Form1->GetPage())]), Form1->TextBuf->Lines->Strings[i]);
            Dummy->ImageIndex = i;
         }
         List->Items->EndUpdate();
     }
     else if (Form1->GetPage() == 0) {
         for (int i=0;i<4;i++) {
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Ground Units");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Air Units");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Heroes");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Buildings");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Addons");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Special Buildings");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Special");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Critters");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Powerups");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Resources");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Special");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Special Buildings");
         }
         for (int i=0;i<GetDATObject(0)->GetDatacount();i++) {
             if (Form1->TextBuf->Lines->Strings[i].Pos("None") != 0) {
	        continue;
             }
             TTreeNode *Dummy;
             if (Form1->GetUseCustomNames() == true) {
   	        Dummy = List->Items->AddChild(List->Items->GetNode(MainNodes[GetOrigin(i,Form1->GetPage())])->Item[GetSubOrigin(i)], LoadName(i));
             }
             else {
                Dummy = List->Items->AddChild(List->Items->GetNode(MainNodes[GetOrigin(i,Form1->GetPage())])->Item[GetSubOrigin(i)], Form1->TextBuf->Lines->Strings[i]);
             }
             Dummy->ImageIndex = i;
         }
         for (int i=0;i<4;i++) {
             for (int j=0;j<List->Items->GetNode(MainNodes[i])->Count;j++) {
                if (List->Items->GetNode(MainNodes[i])->Item[j]->Count == 0) {
                   List->Items->GetNode(MainNodes[i])->Item[j]->Delete();
                   j--;
                }
                else {
                   List->Items->GetNode(MainNodes[i])->Item[j]->ImageIndex = -1;
                }
             }
         }
     }
     List->Items->EndUpdate();
};
//---------------------------------------------------------------------------
/*
############################################################################
Do the same as the above two, but for the 'Sort by Data' sorting option.
Also check and remove the empty categories from the tree view.
############################################################################
*/
void LoadListData (String ListFile, TTreeView* List) {
     List->ClearSelection();
     List->Items->Clear();
     List->Show();
     List->ShowRoot = true;
     List->ShowLines = true;
     List->ShowButtons = true;
     if (Form1->GetPage() == 0) {
        TIniFile *ConfigIni = new TIniFile(Form1->GetProgPath()+"DatEdit.ini");
        Form1->SetUseCustomNames(ConfigIni->ReadBool("Options","UseCustomNames",false));
        delete ConfigIni;
     }
     if (Form1->GetPage() != 0 || (Form1->GetPage() == 0 && Form1->GetUseCustomNames() == false)) {
        Form1->TextBuf->Lines->LoadFromFile(Form1->GetProgPath()+"Data\\"+ListFile+".txt");
     }

     List->Items->BeginUpdate();
     HTREEITEM MainNodes[4];
     MainNodes[0] = (List->Items->AddChild(NULL, "Zerg "+GetDataType(Form1->GetPage())))->ItemId;
     MainNodes[1] = (List->Items->AddChild(NULL, "Terran "+GetDataType(Form1->GetPage())))->ItemId;
     MainNodes[2] = (List->Items->AddChild(NULL, "Protoss "+GetDataType(Form1->GetPage())))->ItemId;
     MainNodes[3] = (List->Items->AddChild(NULL, "Other "+GetDataType(Form1->GetPage())))->ItemId;
     for (int i=0;i<4;i++) {
         List->Items->GetNode(MainNodes[i])->ImageIndex = -1;
     }
     if (Form1->GetPage() != 0) {
         for (int i=0;i<(Form1->TextBuf->Lines->Count);i++) {
            if (Form1->TextBuf->Lines->Strings[i].Pos("None") != 0) {
	       continue;
            }
            TTreeNode* Dummy;
            Dummy=List->Items->AddChild(List->Items->GetNode(MainNodes[GetSortData(i,Form1->GetPage())]), Form1->TextBuf->Lines->Strings[i]);
            Dummy->ImageIndex = i;
         }
         List->Items->EndUpdate();
     }
     else if (Form1->GetPage() == 0) {
         for (int i=0;i<4;i++) {
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Ground Units");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Air Units");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Heroes");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Buildings");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Addons");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Special Buildings");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Special");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Critters");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Powerups");
             List->Items->AddChild(List->Items->GetNode(MainNodes[i]), "Resources");
         }
         for (int i=0;i<GetDATObject(0)->GetDatacount();i++) {
            if (Form1->TextBuf->Lines->Strings[i].Pos("None") != 0) {
	       continue;
            }
            TTreeNode *Dummy;
            if (Form1->GetUseCustomNames() == true) {
               Dummy = List->Items->AddChild(List->Items->GetNode(MainNodes[GetUnitsSortData(i)])->Item[GetUnitsSortSubData(i)], LoadName(i));
            }
            else {
               Dummy = List->Items->AddChild(List->Items->GetNode(MainNodes[GetUnitsSortData(i)])->Item[GetUnitsSortSubData(i)], Form1->TextBuf->Lines->Strings[i]);
            }
            Dummy->ImageIndex = i;
         }

         for (int i=0;i<4;i++) {
             if (List->Items->GetNode(MainNodes[i])->Count == 0) {
                   List->Items->GetNode(MainNodes[i])->Delete();
             }
             else {
                for (int j=0;j<List->Items->GetNode(MainNodes[i])->Count;j++) {
                    if (List->Items->GetNode(MainNodes[i])->Item[j]->Count == 0) {
                       List->Items->GetNode(MainNodes[i])->Item[j]->Delete();
                       j--;
                    }
                    else {
                       List->Items->GetNode(MainNodes[i])->Item[j]->ImageIndex = -1;
                    }
                }
             }
         }
         List->Items->EndUpdate();
     }
};
//---------------------------------------------------------------------------
/*
############################################################################
Perform the backward-reference check for an entry in a DAT, i.e. look
into other DATs to see which entries are pointing to a particular entry,
and output the result in the "UsedBy" list.
############################################################################
*/

void LoadUsedList (int PageIndex,int NodeID) {
     int NoOfEntries;
     int TargetPage;
     int *DataRef;
     bool ListLoaded=false;
     Form1->UsedList->Items->Clear();

//Weapons used by Units
if (PageIndex == 1) {
        DataRef = GetDATObject(0)->GetData();
        NoOfEntries = GetDATObject(0)->GetDatacount();
        for (int i=0;i<NoOfEntries;i++) {
             if (DataRef[i*GetDATObject(0)->GetOffcount()+17] == NodeID || DataRef[i*GetDATObject(0)->GetOffcount()+19] == NodeID) {
                if(ListLoaded == false) {
                     Form1->TextBuf->Lines->LoadFromFile(Form1->GetProgPath()+"Data\\"+GetDATType(0)+".txt");
                     ListLoaded = true;
                }
                Form1->UsedList->Items->Add("[U"+String(i)+"] "+String(Form1->TextBuf->Lines->Strings[i]));
             }
        }
//Weapons used by Orders
        DataRef = GetDATObject(10)->GetData();
        NoOfEntries = GetDATObject(10)->GetDatacount();
        ListLoaded=false;
        for (int i=0;i<NoOfEntries;i++) {
             if (NodeID != 0 && DataRef[i*GetDATObject(10)->GetOffcount()+13] == NodeID) {
                if(ListLoaded == false) {
                     Form1->TextBuf->Lines->LoadFromFile(Form1->GetProgPath()+"Data\\"+GetDATType(10)+".txt");
                     ListLoaded = true;
                }
                Form1->UsedList->Items->Add("[O"+String(i)+"] "+String(Form1->TextBuf->Lines->Strings[i]));
             }
        }
}
//Flingy used by Units
else if (PageIndex == 2) {
        DataRef = GetDATObject(0)->GetData();
        NoOfEntries = GetDATObject(0)->GetDatacount();
        ListLoaded=false;
        for (int i=0;i<NoOfEntries;i++) {
             if (DataRef[i*GetDATObject(0)->GetOffcount()] == NodeID) {
                if(ListLoaded == false) {
                    Form1->TextBuf->Lines->LoadFromFile(Form1->GetProgPath()+"Data\\"+GetDATType(0)+".txt");
                    ListLoaded = true;
                }
                Form1->UsedList->Items->Add("[U"+String(i)+"] "+String(Form1->TextBuf->Lines->Strings[i]));
             }
        }
//Flingy used by Weapons
        DataRef = GetDATObject(1)->GetData();
        NoOfEntries = GetDATObject(1)->GetDatacount();
        ListLoaded=false;
        for (int i=0;i<NoOfEntries;i++) {
             if (DataRef[i*GetDATObject(1)->GetOffcount()+1] == NodeID) {
                if(ListLoaded == false) {
                     Form1->TextBuf->Lines->LoadFromFile(Form1->GetProgPath()+"Data\\"+GetDATType(1)+".txt");
                     ListLoaded = true;
                }
                Form1->UsedList->Items->Add("[W"+String(i)+"] "+String(Form1->TextBuf->Lines->Strings[i]));
             }
        }
}
//Sprites-used-by-Flingy and Images-used-by-Sprites
else if (PageIndex == 3 || PageIndex == 4) {
         DataRef = GetDATObject(PageIndex-1)->GetData();
         NoOfEntries = GetDATObject(PageIndex-1)->GetDatacount();
         ListLoaded=false;
         for (int i=0;i<NoOfEntries;i++) {
             if (DataRef[i*GetDATObject(PageIndex-1)->GetOffcount()] == NodeID) {
                if(ListLoaded == false) {
                    Form1->TextBuf->Lines->LoadFromFile(Form1->GetProgPath()+"Data\\"+GetDATType(PageIndex-1)+".txt");
                    ListLoaded = true;
                }
                switch (PageIndex) {
                       case 3:
                            Form1->UsedList->Items->Add("[F"+String(i)+"] "+String(Form1->TextBuf->Lines->Strings[i]));
                            break;
                       case 4:
                            Form1->UsedList->Items->Add("[S"+String(i)+"] "+String(Form1->TextBuf->Lines->Strings[i]));
                            break;
                }
             }
         }
}
//Upgrades used by Units
else if (PageIndex == 5) {
        DataRef = GetDATObject(0)->GetData();
        NoOfEntries = GetDATObject(0)->GetDatacount();
        ListLoaded=false;
        for (int i=0;i<NoOfEntries;i++) {
             if (DataRef[i*GetDATObject(0)->GetOffcount()+25] == NodeID) {
                if(ListLoaded == false) {
                    Form1->TextBuf->Lines->LoadFromFile(Form1->GetProgPath()+"Data\\"+GetDATType(0)+".txt");
                    ListLoaded = true;
                }
                Form1->UsedList->Items->Add("[U"+String(i)+"] "+String(Form1->TextBuf->Lines->Strings[i]));
             }
        }
//Upgrades used by Weapons
        DataRef = GetDATObject(1)->GetData();
        NoOfEntries = GetDATObject(1)->GetDatacount();
        ListLoaded=false;
        for (int i=0;i<NoOfEntries;i++) {
             if (NodeID != 0 && DataRef[i*GetDATObject(1)->GetOffcount()+6] == NodeID) {
                if(ListLoaded == false) {
                     Form1->TextBuf->Lines->LoadFromFile(Form1->GetProgPath()+"Data\\"+GetDATType(1)+".txt");
                     ListLoaded = true;
                }
                Form1->UsedList->Items->Add("[W"+String(i)+"] "+String(Form1->TextBuf->Lines->Strings[i]));
             }
        }
}
//Techs used by Orders
else if (PageIndex == 6) {
        DataRef = GetDATObject(10)->GetData();
        NoOfEntries = GetDATObject(10)->GetDatacount();
        ListLoaded=false;
        for (int i=0;i<NoOfEntries;i++) {
             if (DataRef[i*GetDATObject(10)->GetOffcount()+14] == NodeID) {
                if(ListLoaded == false) {
                    Form1->TextBuf->Lines->LoadFromFile(Form1->GetProgPath()+"Data\\"+GetDATType(10)+".txt");
                    ListLoaded = true;
                }
                Form1->UsedList->Items->Add("[O"+String(i)+"] "+String(Form1->TextBuf->Lines->Strings[i]));
             }
        }
//Techs used by Weapons
        DataRef = GetDATObject(1)->GetData();
        NoOfEntries = GetDATObject(1)->GetDatacount();
        ListLoaded=false;
        for (int i=0;i<NoOfEntries;i++) {
             if (NodeID != 0 && DataRef[i*GetDATObject(1)->GetOffcount()+2] == NodeID) {
                if(ListLoaded == false) {
                     Form1->TextBuf->Lines->LoadFromFile(Form1->GetProgPath()+"Data\\"+GetDATType(1)+".txt");
                     ListLoaded = true;
                }
                Form1->UsedList->Items->Add("[W"+String(i)+"] "+String(Form1->TextBuf->Lines->Strings[i]));
             }
        }
}
/*
HERE IS AN EXAMPLE OF A DIFFERENT WAY TO DO THE SAME, BUT USING ONE CONDITIONAL
LOOP INSTEAD OF MULTIPLE CONDITIONAL STATEMENTS. I USED THE LATTER OPTION,
BECAUSE IT IS EASIER AND MUCH MORE CLEAR.

     int TargetPage[2];
     int **Modulo;
     int NoOfTargets;
if (PageIndex == 1) {
        NoOfTargets = 2;
        TargetPage[0] = 0;
        TargetPage[1] = 10;
        Modulo = new int*[NoOfTargets];
        Modulo[0] = new int[2];
        Modulo[1] = new int[2];
        Modulo[0][0] = 17;
        Modulo[0][1] = 19;
        Modulo[1][0] = 13;
        Modulo[1][1] = Modulo[1][0];
}
else if (PageIndex == 2) {
        NoOfTargets = 2;
        TargetPage[0] = 0;
        TargetPage[1] = 1;
        Modulo = new int*[NoOfTargets];
        Modulo[0] = new int[2];
        Modulo[1] = new int[2];
        Modulo[0][0] = 0;
        Modulo[0][1] = 0;
        Modulo[1][0] = 1;
        Modulo[1][1] = Modulo[1][0];
}
else if (PageIndex == 3 || PageIndex == 4) {
        NoOfTargets = 1;
        TargetPage[0] = PageIndex-1;
        Modulo = new int*[NoOfTargets];
        Modulo[0] = new int[2];
        Modulo[0][0] = 0;
        Modulo[0][1] = 0;
}
for (int j=0;j<NoOfTargets;j++) {
        DataRef = GetDATObject(TargetPage[j]).GetData();
        NoOfEntries = GetDATObject(TargetPage[j]).GetDatacount();
        for (int i=0;i<NoOfEntries;i++) {
             if (DataRef[i*GetDATObject(TargetPage[j]).GetOffcount()+Modulo[j][0]] == NodeID || DataRef[i*GetDATObject(TargetPage[j]).GetOffcount()+Modulo[j][1]] == NodeID) {
                if(ListLoaded == false) {
                     Form1->TextBuf->Lines->LoadFromFile(Form1->GetProgPath()+"Data\\"+GetDATType(TargetPage[j])+".txt");
                     ListLoaded = true;
                }
                Form1->UsedList->Items->Add("("+String(GetDATType(TargetPage[j])[1]).UpperCase()+String(i)+") "+String(Form1->TextBuf->Lines->Strings[i]));
             }
        }
        ListLoaded = false;
}
*/
};
//---------------------------------------------------------------------------
/*
############################################################################
Check if there are any DAT files present in a designated directory and
load them. Report about the found DATs, no-DATs-found and in case the selected
directory doesn't exist.
############################################################################
*/
void LoadDATDir (String Directory) {
     bool DATFound=false;
     bool FoundDATs[11];

     if (!DirectoryExists(Directory)) {
        MessageDlg(("The : \""+Directory+"\""+" directory does not exist!").c_str(), mtError, TMsgDlgButtons() << mbOK, 0);
     }
     else {
        for (int i=11;i>=0;i--) {
            FoundDATs[i] = false;
            if (FileExists(Directory+"\\"+GetDATType(i)+".dat")) {
               DATFound = true;
               FoundDATs[i] = true;
            }
        }
        if (DATFound == false) {
           MessageDlg(("No DAT files found in: \""+Directory+"\"").c_str(), mtError, TMsgDlgButtons() << mbOK, 0);
        }
        else if (DATFound == true) {
           String Found;
           bool FirstDAT=true;

           ClearMPQImportData();
           for (int i=0;i<11;i++) {
               if (FoundDATs[i] == true) {
                  if (FirstDAT==true) {
                     Found += GetDATType(i)+".dat";
                     FirstDAT = false;
                  }
                  else if (FirstDAT == false) {
                     Found += ", "+GetDATType(i)+".dat";
                  }
               }
           }
           MessageDlg(("DAT types found in \""+Directory+"\":\n"+Found+".").c_str(), mtInformation, TMsgDlgButtons() << mbOK, 0);
           Form1->AddHistoryItem(Directory);
           for (int i=11;i>=0;i--) {
               if (FoundDATs[i] == true) {
                  CheckDATFile(Directory+"\\"+GetDATType(i)+".dat");
               }
           }
        }
     }
};
//---------------------------------------------------------------------------
/*
############################################################################
Load and return a specific entry name from the stat_txt.tbl.
############################################################################
*/

String LoadName (int EntryID) {
     String Name;

     if (Form1->TBLBuf->Tag != 0 || Form1->TBLBuf->Lines->Count == 0) {
        Form1->TBLBuf->Clear();
        if (Form1->GetTBLIsMPQImport(0) == false) {
           ReadTBL(Form1->GetTBL(0), Form1->TBLBuf);
        }
        else {
           ReadTBL(Form1->GetMPQTBL(0), Form1->TBLBuf);
        }
        Form1->TBLBuf->Tag = 0;
     }
     Name = Form1->TBLBuf->Lines->Strings[EntryID];
     Name = Name.SubString(0,Name.Pos("_")-1);
     return Name;
};
//---------------------------------------------------------------------------
