//---------------------------------------------------------------------------
#pragma hdrstop
#include "TBLParser.h"
#include "TBLStatus.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
/*
############################################################################
This is the parsing routine for the TBL files. It outputs the content of a
given TBL file (FileName) to a TMemo-type control.
A few notes of explanation:
1)the 'address' array stores the starting offsets of the strings,
2)the 'length' array stores the length of the strings,
3)the 'data' array stores all the strings from the parsed TBL
############################################################################
*/

void ReadTBL (String FileName, TMemo* OutputMemo)
{
int size;
unsigned short *address;
short count,*length;
char *data;
void *buf;

if (FileExists(FileName)) {
     TFileStream *TBLStream = new TFileStream(FileName,fmOpenRead);

     OutputMemo->Clear();

     size = TBLStream->Seek(0,soFromEnd);
     TBLStream->Seek(0,soFromBeginning);
     TBLStream->Read(&buf, 2);
     count = short (buf);

     address = new unsigned short [count];
     length = new short [count];

     TBLStream->Seek(2,soFromBeginning);
     TBLStream->Read(&buf, 2);
     address[0] = int (buf);

     for (int i=0;i<count;i++) {
         TBLStream->Seek(2+(i*2),soFromBeginning);
         TBLStream->Read(&buf,2);
         address[i] = (unsigned short) buf;
         if (i>=1) {
            if (i+1==count) {
	       length[i]=size-address[i];
            }
            length[i-1]=address[i]-address[i-1]-1;
         }
     }

     data = new char [(size-address[0])];
     for (int i=0,k=0;i<count;i++) {
	 for (int j=0;j<=length[i] && k<(size-address[0]);j++,k++) {
             TBLStream->Seek(address[i]+j,soFromBeginning);
             TBLStream->Read(&buf,1);
             data[k] = char (buf);
             if (data[k]==0x0) {
                if (j+1>=length[i]) {
                   data[k]='\n';
                }
                else if (j+1<length[i]) {
    		   data[k]='_';
                }
             }
             else if (data[k]==0xA) {
                data[k]='|';
             }
             else if (data[k]==0x20) {
                data[k]=' ';
             }
        }
     }      
     OutputMemo->Lines->Text = String(data);
     if (OutputMemo->Lines->Count > count) {
        while (OutputMemo->Lines->Count > count) {
              OutputMemo->Lines->Delete(OutputMemo->Lines->Count-1);
        }
     }
     delete TBLStream;
}
else {
    MessageDlg(String(FileName+" not found").c_str(), mtError, TMsgDlgButtons() << mbOK, 0);
}
};
//---------------------------------------------------------------------------

